"""
Remediate Evident Signature:

AWS:RDS-005 RDS Database Publicly Accessible

Description:

RDS database instances should not be made accessible to the public internet, outside of your VPC. This is
dangerous, as RDS databases should normally be privately accessible only from within your VPC.

Required Permissions:

- rds:DescribeDBInstances
- rds:ModifyDBInstance

Sample IAM Policy:

{
  "Version": "2012-10-17",
  "Statement": [
    {
      "Sid": "RDSPermissions",
      "Action": [
        "rds:DescribeDBInstances",
        "rds:ModifyDBInstance"
      ],
      "Effect": "Allow",
      "Resource": "*"
    }
  ]
}
"""

import boto3
from botocore.exceptions import ClientError


def remediate(session, alert, lambda_context):
  """
  Main Function invoked by index.py
  """

  instance_id = alert['resource_id']
  region = alert['region']

  rds = session.client('rds', region_name=region)

  try:
    db_instance = rds.describe_db_instances(DBInstanceIdentifier = instance_id)['DBInstances']
  except ClientError as e:
    print(e.response['Error']['Message'])
    return

  try:
    public = db_instance[0]['PubliclyAccessible']
  except (KeyError, IndexError):
    public = False

  if public == True: 

    try:
      rds.modify_db_instance(
        DBInstanceIdentifier = instance_id,
        PubliclyAccessible = False
      )
    except ClientError as e:
      print(e.response['Error']['Message'])
      return

    else:
      print('Removed public attribute from RDS instance {}.'.format(instance_id))

  return




"""
Remediate Evident Signature:

AWS:CONFIG-001 AWS Config is not enabled

Description:

The AWS Config service provides visibility to user configuration history and configuration change
notifications. Along with use of CloudTrail, we consider enabling AWS Config to be a best practice for users.

**Note: No Config Rules are created.

Required Permissions:

- iam:AttachRolePolicy
- iam:CreateRole
- iam:GetRole
- s3:CreateBucket
- s3:PutBucketPolicy
- config:PutConfigurationRecorder
- config:PutDeliveryChannel
- config:StartConfigurationRecorder

Sample IAM Policy:

{
  "Version": "2012-10-17",
  "Statement": [
    {
      "Sid": "IAMPermissions",
      "Action": [
        "iam:AttachRolePolicy",
        "iam:CreateRole",
        "iam:GetRole"
      ],
      "Effect": "Allow",
      "Resource": "*"
    },
    {
      "Sid": "S3Permissions",
      "Action": [
        "s3:CreateBucket",
        "s3:PutBucketPolicy"
      ],
      "Effect": "Allow",
      "Resource": "*"
    },
    {
      "Sid": "ConfigPermissions",
      "Action": [
        "config:PutConfigurationRecorder",
        "config:PutDeliveryChannel",
        "config:StartConfigurationRecorder"
      ],
      "Effect": "Allow",
      "Resource": "*"
    }
  ]
}
"""

import json
import boto3
from botocore.exceptions import ClientError
from time import sleep


def remediate(session, alert, lambda_context):
  """
  Main Function invoked by index.py
  """

  region = alert['region']

  iam  = session.client('iam', region_name=region)
  s3   = session.client('s3', region_name=region)
  config = session.client('config', region_name=region)

  # Create IAM Role
  role_arn, account_id = new_iam_role(iam, region)

  # Create Config Recorder
  recorder_name = new_config_recorder(config, role_arn) if (role_arn != 'fail') else 'fail'

  # Create S3 Bucket
  bucket_name   = new_s3_bucket(s3, account_id, region) if (role_arn != 'fail') else 'fail'

  # Create Config Delivery Channel
  channel_name  = new_config_channel(config, bucket_name) if (bucket_name != 'fail') else 'fail'

  # Start Config Recorder
  start_result  = start_recorder(config, recorder_name) if (channel_name != 'fail' and recorder_name != 'fail') else 'fail'

  # Results
  if start_result != 'fail':
    print('AWS Config enabled in region {}.'.format(region))
  else:
    print('Failed to enable AWS Config in region {}.'.format(region))

  return


def new_iam_role(iam, region):
  """
  Create new IAM Role and attach AWS Config Managed Policy
  """

  role_name = 'config-role-' + region

  try:
    role = iam.create_role(
      Path = '/',
      RoleName = role_name,
      AssumeRolePolicyDocument = json.dumps(RoleTemplate.RolePolicy)
    )
    role_arn = role['Role']['Arn']

    print('New IAM Role created: {}'.format(role_arn))
    sleep(10)  # Wait for IAM resource to be available. >> Gotta be a better way.. { wait? get? }.

  except ClientError as e:
    if e.response['Error']['Code'] == 'EntityAlreadyExists':
      role = iam.get_role(
        RoleName = role_name
      )
      role_arn = role['Role']['Arn']

      print('Using existing IAM Role: {}'.format(role_arn))

    else:
      print(e.response['Error']['Message'])
      return 'fail', None

  try:
    result = iam.attach_role_policy(
      RoleName = role_name,
      PolicyArn = 'arn:aws:iam::aws:policy/service-role/AWSConfigRole'
    )
  except ClientError as e:
    print(e.response['Error']['Message'])
    return 'fail', None

  account_id = role_arn.split(':')[4]

  return role_arn, account_id


def new_s3_bucket(s3, account_id, region):
  """
  Create new S3 Bucket
  """

  bucket_name = 'config-bucket-' + account_id

  try:
    if region == 'us-east-1':
      result = s3.create_bucket(
        ACL = 'private',
        Bucket = bucket_name
      )
    else:
      result = s3.create_bucket(
        ACL = 'private',
        Bucket = bucket_name,
        CreateBucketConfiguration = {'LocationConstraint': region}
      )

    print('New S3 bucket created: {}'.format(bucket_name))

  except ClientError as e:
    if e.response['Error']['Code'] == 'BucketAlreadyExists':
      print('Using existing S3 bucket: {}'.format(bucket_name))
    elif e.response['Error']['Code'] == 'BucketAlreadyOwnedByYou':
      print('Using already owned and existing S3 bucket: {}'.format(bucket_name))
    else:
      print(e.response['Error']['Message'])
      return 'fail'

  try:
    result = s3.put_bucket_policy(
      Bucket = bucket_name,
      Policy = json.dumps(BucketTemplate.BucketPolicy(bucket_name, account_id))
    )
  except ClientError as e:
      print(e.response['Error']['Message'])
      return 'fail'

  return bucket_name
  

def new_config_recorder(config, role_arn):
  """
  Create new Config Recorder
  """

  recorder_name = 'default'

  try:
    result = config.put_configuration_recorder(
      ConfigurationRecorder = {
        'name': recorder_name,
        'roleARN': role_arn,
        'recordingGroup': {
            'allSupported': True
        }
      }
    )

    print('New Config recorder created: {}'.format(recorder_name))

  except ClientError as e:
    print(e.response['Error']['Message'])
    return 'fail'

  return recorder_name


def new_config_channel(config, bucket_name):
  """
  Create new Config Delivery Channel
  """

  channel_name = 'default'

  try:
    result = config.put_delivery_channel(
      DeliveryChannel = {
        'name': channel_name,
        's3BucketName': bucket_name,
        'configSnapshotDeliveryProperties': {
          'deliveryFrequency': 'One_Hour'
        }
      }
    )

    print('New Config delivery channel created: {}'.format(channel_name))

  except ClientError as e:
    print(e.response['Error']['Message'])
    return 'fail'

  return channel_name


def start_recorder(config, recorder_name):
  """
  Start Config Recorder
  """

  try:
    result = config.start_configuration_recorder(
      ConfigurationRecorderName = recorder_name
    )
  except ClientError as e:
    print(e.response['Error']['Message'])
    return 'fail'

  return


class RoleTemplate:

  RolePolicy = {
                 'Version': '2012-10-17',
                 'Statement': [
                   {
                     'Sid': '',
                     'Effect': 'Allow',
                     'Principal': {
                       'Service': 'config.amazonaws.com'
                     },
                     'Action': 'sts:AssumeRole'
                   }
                 ]
               }

class BucketTemplate():

  def BucketPolicy(bucket_name, account_id):

    Policy = {
               'Version': '2012-10-17',
               'Statement': [
                 {
                   'Sid': 'AWSConfigBucketPermissionsCheck',
                   'Effect': 'Allow',
                   'Principal': {
                     'Service': [
                       'config.amazonaws.com'
                     ]
                   },
                   'Action': 's3:GetBucketAcl',
                   'Resource': 'arn:aws:s3:::' + bucket_name
                 },
                 {
                   'Sid': ' AWSConfigBucketDelivery',
                   'Effect': 'Allow',
                   'Principal': {
                     'Service': [
                       'config.amazonaws.com'    
                     ]
                   },
                   'Action': 's3:PutObject',
                   'Resource': 'arn:aws:s3:::' + bucket_name + '/AWSLogs/' + account_id + '/Config/*',
                   'Condition': { 
                     'StringEquals': { 
                       's3:x-amz-acl': 'bucket-owner-full-control' 
                     }
                   }
                 }
               ]
             }   

    return Policy




"""
Remediate Evident Signature:

AWS:SSS-009 S3 Logging Enabled

Description:

S3 Logging provides a way for you to get details about S3 bucket activity. By default, AWS does not enable
logging for S3 buckets. This additional insight into bucket activity can be useful when troubleshooting and
may be requested by AWS Support. Logging cannot be enabled retroactive to an issue.

Required Permissions:

- sts:GetCallerIdentity
- s3:CreateBucket
- s3:GetBucketLogging
- s3:PutBucketLogging

Sample IAM Policy:

{
  "Version": "2012-10-17",
  "Statement": [
    {
      "Sid": "STSPermissions",
      "Action": [
        "sts:GetCallerIdentity"
      ],
      "Effect": "Allow",
      "Resource": "*"
    },
    {
      "Sid": "S3Permissions",
      "Action": [
        "s3:CreateBucket",
        "s3:GetBucketLogging",
        "s3:PutBucketLogging"
      ],
      "Effect": "Allow",
      "Resource": "*"
    }
  ]
}
"""

import boto3
from botocore.exceptions import ClientError


def remediate(session, alert, lambda_context):
  """
  Main Function invoked by index.py
  """

  bucket_name = alert['resource_id']
  region = alert['region']

  s3  = session.client('s3', region_name=region)
  sts = session.client('sts', region_name=region)

  try:
    logging = s3.get_bucket_logging(Bucket=bucket_name)
  except ClientError as e:
    print(e.response['Error']['Message'])
    return

  # Is bucket logging already enabled?
  try:
    enabled = logging['LoggingEnabled']
  except KeyError:
    enabled = None

  if enabled != None:
    print('Logging already enabled for S3 bucket: {}.'.format(bucket_name))
    return

  # Grab the AWS account Id
  account_id = get_account_id(sts)

  # Create new target bucket OR return an existing one
  target_bucket = new_s3_bucket(s3, account_id, region) if (account_id != 'fail') else 'fail'

  # Enable bucket logging
  if target_bucket != 'fail':
    result = update_s3_bucket(s3, bucket_name, target_bucket)

  return


def get_account_id(sts):
  """
  Return AWS Account Id
  """

  try:
    account_id = sts.get_caller_identity()['Account']
  except ClientError as e:
    print(e.response['Error']['Message'])
    return 'fail'

  return account_id


def new_s3_bucket(s3, account_id, region):
  """
  Create new S3 target bucket OR return an existing one
  """

  target_bucket = 's3accesslogs-' + account_id + '-' + region

  try:
    if region == 'us-east-1':
      result = s3.create_bucket(
        ACL = 'log-delivery-write',
        Bucket = target_bucket
      )
    else:
      result = s3.create_bucket(
        ACL = 'log-delivery-write',
        Bucket = target_bucket,
        CreateBucketConfiguration = {'LocationConstraint': region}
      )

    print('New S3 target bucket created: {}'.format(target_bucket))

  except ClientError as e:
    if e.response['Error']['Code'] == 'BucketAlreadyExists':
      print('Using existing S3 target bucket: {}'.format(target_bucket))
    elif e.response['Error']['Code'] == 'BucketAlreadyOwnedByYou':
      print('Using already owned and existing S3 target bucket: {}'.format(target_bucket))
    else:
      print(e.response['Error']['Message'])
      return 'fail'

  return target_bucket


def update_s3_bucket(s3, bucket_name, target_bucket):
  """
  Enable S3 bucket logging
  """

  try:
    result = s3.put_bucket_logging(
      Bucket = bucket_name,
      BucketLoggingStatus = {
        'LoggingEnabled': {
          'TargetBucket': target_bucket,
          'TargetPrefix': ''
        }
      }
    )
  except ClientError as e:
    print(e.response['Error']['Message'])

  else:
    print('Logging enabled for S3 bucket: {}.'.format(bucket_name))

  return




"""
Remediate Evident Signature:

AWS:IAM-018 IAM Support Role Check

Description:

The best security practice is to assign the least privilege available for access control to a user's role
to manage incidents with AWS Support. "AWSSupportAccess" is the default policy for AWS.

Required Permissions:

- iam:AttachRolePolicy
- iam:CreateRole
- iam:CreateUser
- iam:GetPolicy

Sample IAM Policy:

{
  "Version": "2012-10-17",
  "Statement": [
    {
      "Sid": "IAMPermissions",
      "Action": [
        "iam:AttachRolePolicy",
        "iam:CreateRole",
        "iam:CreateUser",
        "iam:GetPolicy"
      ],
      "Effect": "Allow",
      "Resource": "*"
    }
  ]
}
"""

import json
import boto3
from botocore.exceptions import ClientError
from time import sleep

# Options:
#
# Default User, Role and Policy
#
support_role_name  = 'AWSSupportRole'
support_user_name  = 'AWSSupportUser'
support_policy_arn = 'arn:aws:iam::aws:policy/AWSSupportAccess'


def remediate(session, alert, lambda_context):
  """
  Main Function invoked by index.py
  """

  region = alert['region']

  iam = session.client('iam', region_name=region)

  try:
    policy = iam.get_policy(PolicyArn = support_policy_arn)['Policy']
  except ClientError as e:
    print(e.response['Error']['Message'])
    return 'fail'

  if policy['AttachmentCount'] <= 0:

    # Create IAM User
    user_arn = new_iam_user(iam)

    # Create IAM Role
    role_arn = new_iam_role(iam, user_arn) if (user_arn != 'fail') else 'fail'

    # Result
    if role_arn != 'fail':
      print('A Support Role has been created to manage incidents with AWS Support.')
    else:
      print('Failed to create a Support Role to manage incidents with AWS Support.')

  else:
    print('AWS Support policy has one or more attachments: {}'.format(support_policy_arn))

  return


def new_iam_user(iam):
  """
  Create new Support IAM User
  """

  try:
    user = iam.create_user(
      Path = '/',
      UserName = support_user_name
    )
  except ClientError as e:
    print(e.response['Error']['Message'])
    return 'fail'

  else:
    user_arn = user['User']['Arn']
    print('New IAM Support User created: {}'.format(user_arn))

    sleep(10)  # Wait for IAM resource to be available. >> Gotta be a better way.. { wait? get? }.

  return user_arn


def new_iam_role(iam, user_arn):
  """
  Create new Support IAM Role and attach AWS Managed Policy
  """

  try:
    role = iam.create_role(
      Path = '/',
      RoleName = support_role_name,
      AssumeRolePolicyDocument = json.dumps(Template.RolePolicy(user_arn))
    )
  except ClientError as e:
    print(e.response['Error']['Message'])
    return 'fail'

  else:
    role_arn  = role['Role']['Arn']
    print('New IAM Support Role created: {}'.format(role_arn))

  try:
    result = iam.attach_role_policy(
      RoleName = support_role_name,
      PolicyArn = support_policy_arn
    )
  except ClientError as e:
    print(e.response['Error']['Message'])
    return 'fail'
    
  return role_arn


class Template():

  def RolePolicy(user_arn):

    Policy =  {
                'Version': '2008-10-17',
                'Statement': [
                  {
                    'Effect': 'Allow',
                    'Principal': {
                      'AWS': user_arn
                    },
                    'Action': 'sts:AssumeRole'
                  }
                ]
              }

    return Policy




"""
Remediate Evident Signature:

AWS:IAM-015 Unused IAM Access Keys

Description:

IAM users can access AWS resources using different types of credentials, such as passwords or access keys.
It is recommended that all credentials that have been unused in 90 or greater days be removed or deactivated.

Required Permissions:

- iam:GetAccessKeyLastUsed
- iam:UpdateAccessKey

Sample IAM Policy:

{
  "Version": "2012-10-17",
  "Statement": [
    {
      "Sid": "IAMPermissions",
      "Action": [
        "iam:GetAccessKeyLastUsed",
        "iam:UpdateAccessKey"
      ],
      "Effect": "Allow",
      "Resource": "*"
    }
  ]
}
"""

import boto3
from botocore.exceptions import ClientError
from datetime import datetime
from datetime import date


def remediate(session, alert, lambda_context):
  """
  Main Function invoked by index.py
  """

  key_id = alert['resource_id']
  region = alert['region']

  iam = session.client('iam', region_name=region)

  try:
    resp = iam.get_access_key_last_used(AccessKeyId=key_id)
  except ClientError as e:
    print(e.response['Error']['Message'])
    return

  user_name = resp['UserName']
  last_used = resp['AccessKeyLastUsed']['LastUsedDate'].date()

  today = date.today()
  delta = today - last_used

  if delta.days >= 90:
    result = deactivate_access_key(iam, key_id, user_name) 

  return


def deactivate_access_key(iam, key_id, user_name):
  """
  Deactivate IAM Access Key
  """

  try:
    result = iam.update_access_key(
      UserName = user_name,
      AccessKeyId = key_id,
      Status = 'Inactive'
    )
  except ClientError as e:
    print(e.response['Error']['Message'])

  else:
    print('Deactivated access key {} for user {}.'.format(key_id, user_name))

  return




"""
Remediate Evident Signature:

AWS:KMS-001 KMS Key Rotation Disabled

Description:

The recommended best security practice is to allow for the KMS rotation of a Customer Master Key (CMK).

Enabling the rotation of CMKs will create a new version of the backing key for each rotation. Evident
recommends that the CMK keys are rotated every year.

Required Permissions:

- kms:EnableKeyRotation

Sample IAM Policy:

{
  "Version": "2012-10-17",
  "Statement": [
    {
      "Sid": "KMSPermissions",
      "Action": [
        "kms:EnableKeyRotation"
      ],
      "Effect": "Allow",
      "Resource": "*"
    }
  ]
}
"""

import boto3
from botocore.exceptions import ClientError


def remediate(session, alert, lambda_context):
  """
  Main Function invoked by index.py
  """

  key_id = alert['resource_id']
  region = alert['region']

  kms = session.client('kms', region_name=region)

  try:
    result = kms.enable_key_rotation(KeyId = key_id)
  except ClientError as e:
    print(e.response['Error']['Message'])
    return

  else:
    print('KMS key rotation enabled for Customer Master Key: {}.'.format(key_id))

  return




"""
Remediate Evident Signature:

AWS:REDSHIFT-001 Redshift Cluster Is Publicly Accessible

Description:

It is the recommended best practice that Redshift cluster nodes are not accessible to the public internet,
outside of your VPC. This is dangerous, as Redshift databases should normally be privately accessible only
from within your VPC.

Required Permissions:

- redshift:DescribeClusters
- redshift:ModifyCluster

Sample IAM Policy:

{
  "Version": "2012-10-17",
  "Statement": [
    {
      "Sid": "RedshiftPermissions",
      "Action": [
        "redshift:DescribeClusters",
        "redshift:ModifyCluster"
      ],
      "Effect": "Allow",
      "Resource": "*"
    }
  ]
}
"""

import boto3
from botocore.exceptions import ClientError


def remediate(session, alert, lambda_context):
  """
  Main Function invoked by index.py
  """

  cluster_id = alert['resource_id']
  region = alert['region']

  redshift = session.client('redshift', region_name=region)

  try:
    cluster = redshift.describe_clusters(ClusterIdentifier=cluster_id)['Clusters']
  except ClientError as e:
    print(e.response['Error']['Message'])
    return

  try:
    public = cluster[0]['PubliclyAccessible']
  except (KeyError, IndexError):
    public = False

  if public == True: 

    try:
      redshift.modify_cluster(
        ClusterIdentifier = cluster_id,
        PubliclyAccessible = False
      )
    except ClientError as e:
      print(e.response['Error']['Message'])
      return

    else:
      print('Removed public attribute from Redshift cluster {}.'.format(cluster_id))

  return




"""
Remediate Evident Signature:

AWS:EC2-039 Global Access on All Ports Detected

Description:

Global permission to access all ports should not be allowed in a security group. Best security practice
dictates restricting access to service ports solely to known static IP addresses within your control.

Required Permissions:

- ec2:DescribeSecurityGroups
- ec2:RevokeSecurityGroupIngress

Sample IAM Policy:

{
    "Version": "2012-10-17",
    "Statement": [
        {
            "Sid": "EC2Permissions",
            "Effect": "Allow",
            "Action": [
                "ec2:DescribeSecurityGroups",
                "ec2:RevokeSecurityGroupIngress"
            ],
            "Resource": [
                "*"
            ]
        }
    ]
}
"""

import boto3
from botocore.exceptions import ClientError

# Options:
#
global_cidr_list = [ '0.0.0.0/0', '::/0' ]


def remediate(session, alert, lambda_context):
  """
  Main Function invoked by index.py
  """

  sg_id  = alert['resource_id']
  region = alert['region']

  ec2   = session.client('ec2', region_name=region)

  try:
    group = ec2.describe_security_groups(GroupIds=[ sg_id, ])['SecurityGroups']
  except ClientError as e:
    print(e.response['Error']['Message'])
    return

  try:
    ip_perms = group[0]['IpPermissions']
  except (IndexError, KeyError):
    print('IP permissions not found for security group {}.'.format(sg_id))
    return

  for ip_perm in ip_perms:
    result = remove_offending_sg_rules(ec2, sg_id, ip_perm)

  return


def remove_sg_rule(ec2, revoke_args):
  """
  Revoke ingress security group rule
  """

  try:
    ec2.revoke_security_group_ingress(**revoke_args)
  except ClientError as e:
    print(e.response['Error']['Message'])
    return

  else:
    print('Revoked security group rule: {}.'.format(revoke_args))

  return


def remove_offending_sg_rules(ec2, sg_id, ip_perm):
  """
  Remove offending security group rules (calls >> remove_sg_rule())
  """

  # Look for IPv4 permissions
  if ip_perm['IpRanges']:

    for ip_range in ip_perm['IpRanges']:
      if ip_range['CidrIp'] in global_cidr_list:

        if (ip_perm['IpProtocol'] == '-1'):
          revoke_args = {
            'GroupId' : sg_id,
            'IpPermissions' : [
              {
                'IpProtocol': ip_perm['IpProtocol'],
                'IpRanges': [{ 'CidrIp': ip_range['CidrIp'] }]
              }
            ]
          }

        elif (ip_perm['FromPort'] == 0 and ip_perm['ToPort'] == 65535):
          revoke_args = {
            'GroupId' : sg_id,
            'IpPermissions' : [
              {
                'IpProtocol': ip_perm['IpProtocol'],
                'FromPort': ip_perm['FromPort'],
                'ToPort': ip_perm['ToPort'],
                'IpRanges': [{ 'CidrIp': ip_range['CidrIp'] }]
              }
            ]
          }
        else:
          revoke_args = None

        if revoke_args != None:
          result = remove_sg_rule(ec2, revoke_args)

  # Look for IPv6 permissions
  if ip_perm['Ipv6Ranges']:

    for ip_range in ip_perm['Ipv6Ranges']:
      if ip_range['CidrIpv6'] in global_cidr_list:

        if (ip_perm['IpProtocol'] == '-1'):
          revoke_args = {
            'GroupId' :  sg_id,
            'IpPermissions' : [
              {
                'IpProtocol': ip_perm['IpProtocol'],
                'Ipv6Ranges': [{ 'CidrIpv6': ip_range['CidrIpv6'] }]
              }
            ]
          }

        elif (ip_perm['FromPort'] == 0 and ip_perm['ToPort'] == 65535):
          revoke_args = {
            'GroupId' :  sg_id,
            'IpPermissions' : [
              {
                'IpProtocol': ip_perm['IpProtocol'],
                'FromPort': ip_perm['FromPort'],
                'ToPort': ip_perm['ToPort'],
                'Ipv6Ranges': [{ 'CidrIpv6': ip_range['CidrIpv6'] }]
              }
            ]
          }
        else:
          revoke_args = None

        if revoke_args != None:
          result = remove_sg_rule(ec2, revoke_args)

  return




"""
Remediate Evident Signature:

AWS:EC2-002 Global Admin Port Access - SSH (TCP Port 22) Detected

Description:

Global permission to access the well known services TCP port 22 (SSH) should not be allowed in a
security group.

**Note: Remediation will be executed if the well know service is found within a port range.

Required Permissions:

- ec2:DescribeSecurityGroups
- ec2:RevokeSecurityGroupIngress

Sample IAM Policy:

{
    "Version": "2012-10-17",
    "Statement": [
        {
            "Sid": "EC2Permissions",
            "Effect": "Allow",
            "Action": [
                "ec2:DescribeSecurityGroups",
                "ec2:RevokeSecurityGroupIngress"
            ],
            "Resource": [
                "*"
            ]
        }
    ]
}
"""

import json
import re
import boto3
from botocore.exceptions import ClientError

# Options:
#
# Example
#   admin_port_list = [ 'tcp-22', 'tcp-23', 'tcp-3389' ]
#
admin_port_list  = [ 'tcp-22' ]
global_cidr_list = [ '0.0.0.0/0', '::/0' ]


def remediate(session, alert, lambda_context):
  """
  Main Function invoked by index.py
  """

  sg_id  = alert['resource_id']
  region = alert['region']

  ec2   = session.client('ec2', region_name=region)

  try:
    group = ec2.describe_security_groups(GroupIds=[ sg_id, ])['SecurityGroups']
  except ClientError as e:
    print(e.response['Error']['Message'])
    return

  try:
    ip_perms = group[0]['IpPermissions']
  except (IndexError, KeyError):
    print('IP permissions not found for security group {}.'.format(sg_id))
    return

  for ip_perm in ip_perms:
    try:
      from_port   = ip_perm['FromPort']
    except KeyError:
      continue
    else:
      to_port     = ip_perm['ToPort']
      ip_protocol = ip_perm['IpProtocol']

    # Look for IPv4 permissions
    if ip_perm['IpRanges']:
      IpRanges = 'IpRanges'
      IpCidr   = 'CidrIp'

      for ip_range in ip_perm['IpRanges']:
        cidr_ip = ip_range['CidrIp']
        remove_sg_rule(ec2, sg_id, from_port, to_port, ip_protocol, cidr_ip, IpRanges, IpCidr)

    # Look for IPv6 permissions
    if ip_perm['Ipv6Ranges']:
      IpRanges = 'Ipv6Ranges'
      IpCidr   = 'CidrIpv6'

      for ip_range in ip_perm['Ipv6Ranges']:
        cidr_ip = ip_range['CidrIpv6']
        remove_sg_rule(ec2, sg_id, from_port, to_port, ip_protocol, cidr_ip, IpRanges, IpCidr)

  return


def remove_sg_rule(ec2, sg_id, from_port, to_port, ip_protocol, cidr_ip, IpRanges, IpCidr):
  """
  Revoke Ingress Security Group Rule
  """

  for admin_port in admin_port_list:
    proto = re.split('-', admin_port)[0]
    port  = re.split('-', admin_port)[1]

    find_port='true' if from_port <= int(port) <= to_port else 'false'

    if cidr_ip in global_cidr_list and ip_protocol.lower() == proto and find_port == 'true':

      revoke_args = {
        'GroupId' :  sg_id,
        'IpPermissions' : [
          {
            'IpProtocol': ip_protocol,
            'FromPort': from_port,
            'ToPort': to_port,
             IpRanges: [{ IpCidr: cidr_ip }]
          }
        ]
      }

      try:
        ec2.revoke_security_group_ingress(**revoke_args)

        print('Revoked rule permitting {}/{:d}-{:d} with cidr {} from {}.'.format(ip_protocol, from_port, to_port, cidr_ip, sg_id))

      except ClientError as e:
        print(e.response['Error']['Message'])

  return



"""
Remediate Evident Signature:

AWS:CLT-002 CloudTrail log not encrypted with SSE-KMS

Description:

CloudTrail configured to use SSE-KMS provides additional confidentiality controls on log data. To gain
access, users must have S3 read permission on the corresponding log bucket and must be granted decrypt
permission by the customer-created master keys (CMK) policy.

Required Permissions:

- cloudtrail:DescribeTrails
- cloudtrail:UpdateTrail
- s3:GetBucketLocation
- kms:CreateAlias
- kms:CreateKey

Sample IAM Policy:

{
  "Version": "2012-10-17",
  "Statement": [
    {
      "Sid": "CloudTrailPermissions",
      "Action": [
        "cloudtrail:DescribeTrails",
        "cloudtrail:UpdateTrail"
      ],
      "Effect": "Allow",
      "Resource": "*"
    },
    {
      "Sid": "S3Permissions",
      "Action": [
        "s3:GetBucketLocation"
      ],
      "Effect": "Allow",
      "Resource": "*"
    },
    {
      "Sid": "KMSPermissions",
      "Action": [
        "kms:CreateAlias",
        "kms:CreateKey"
      ],
      "Effect": "Allow",
      "Resource": "*"
    }
  ]
}
"""

import json
import boto3
from botocore.exceptions import ClientError


def remediate(session, alert, lambda_context):
  """
  Main Function invoked by index.py
  """

  trail_name = alert['resource_id']
  region     = alert['region']

  clt = session.client('cloudtrail', region_name=region)
  s3  = session.client('s3', region_name=region)

  try:
    trail = clt.describe_trails(trailNameList=[ trail_name ], includeShadowTrails=False)['trailList']
  except ClientError as e:
    print(e.response['Error']['Message'])
    return

  if not trail:
    print('Error: Unable to find Trail {}.'.format(trail_name))
    return

  # Check the Trail home region
  try:
    home_region = trail[0]['HomeRegion']
  except (KeyError, IndexError):
    print('Error: Unable to determine the home region for Trail {}.'.format(trail_name))
    return

  else:
    if home_region != region:
      return

  # Is the Trail KmsKeyId in-use?
  try:
    trail_key = trail[0]['KmsKeyId']
  except (KeyError, IndexError):
    trail_key = None

  if trail_key == None:

    # Grab the Trail Arn (we need the Account Id)
    try:
      trail_arn  = trail[0]['TrailARN']
    except (KeyError, IndexError):
      print('Error: Unable to find the ARN for Trail {}.'.format(trail_name))
      return

    else:
      account_id = trail_arn.split(':')[4]

    # Grab the Trail S3 logging bucket
    try:
      trail_bucket = trail[0]['S3BucketName']
    except (KeyError, IndexError):
      print('Error: Unable to find the S3 logging bucket for Trail {}.'.format(trail_name))
      return

    # Find the Trail S3 logging bucket location (region)
    # More info: https://docs.aws.amazon.com/general/latest/gr/rande.html#s3_region
    try:
      bucket_region = s3.get_bucket_location(Bucket=trail_bucket)['LocationConstraint']
    except ClientError as e:
      print(e.response['Error']['Message'])
      return

    if not bucket_region: bucket_region = 'us-east-1'     # N. Virginia
    if bucket_region == 'EU': bucket_region = 'eu-west-1' # Ireland

    key_id = create_cmk(account_id, trail_name, bucket_region)

    if key_id != 'fail':
      result = update_trail_cmk(clt, trail_name, key_id)

  return


def update_trail_cmk(clt, trail_name, key_id):
  """
  Update Trail with CMK
  """

  try:
    result = clt.update_trail(
      Name = trail_name,
      KmsKeyId = key_id
    )
  except ClientError as e:
    print(e.response['Error']['Message'])

  else:
    print('Updated Trail {} with KMS Customer Managed encryption Key: {}.'.format(trail_name, key_id))

  return


def create_cmk(account_id, trail_name, bucket_region):
  """
  Create Customer Managed Key (CMK)
  """

  # Create KMS client session in the same region as the Trail S3 logging bucket
  kms = session.client('kms', region_name=bucket_region)

  cmk_alias_name = 'alias/' + trail_name + '-cloudtrail-key'

  # Create CMK
  try:
    cmk = kms.create_key(
      Description = 'CMK for CloudTrail Logs',
      KeyUsage = 'ENCRYPT_DECRYPT',
      Origin = 'AWS_KMS',
      BypassPolicyLockoutSafetyCheck = True,
      Policy = json.dumps(KMSTemplate.CMKPolicy(account_id, bucket_region))
    ) 
  except ClientError as e:
    print(e.response['Error']['Message'])
    return 'fail'

  else:
    key_id = cmk['KeyMetadata']['Arn']

  # Create CMK alias
  try:
    result = kms.create_alias(
      AliasName = cmk_alias_name,
      TargetKeyId = key_id
    )
  except ClientError as e:
    print(e.response['Error']['Message'])

  return key_id


class KMSTemplate():

  def CMKPolicy(account_id, bucket_region):

    Policy = {
               'Version': '2012-10-17',
               'Id': 'Key policy created by CloudTrail',
               'Statement': [
                 {
                   'Sid': 'Enable IAM User Permissions',
                   'Effect': 'Allow',
                   'Principal': {
                     'AWS': [
                       'arn:aws:iam::' + account_id + ':root'
                     ]
                   },
                   'Action': 'kms:*',
                   'Resource': '*'
                 },
                 {
                   'Sid': 'Allow CloudTrail to encrypt logs',
                   'Effect': 'Allow',
                   'Principal': {
                     'Service': 'cloudtrail.amazonaws.com'
                   },
                   'Action': 'kms:GenerateDataKey*',
                   'Resource': '*',
                   'Condition': {
                     'StringLike': {
                       'kms:EncryptionContext:aws:cloudtrail:arn': 'arn:aws:cloudtrail:*:' + account_id + ':trail/*'
                     }
                   }
                 },
                 {
                   'Sid': 'Allow CloudTrail to describe key',
                   'Effect': 'Allow',
                   'Principal': {
                     'Service': 'cloudtrail.amazonaws.com'
                   },
                   'Action': 'kms:DescribeKey',
                   'Resource': '*'
                 },
                 {
                   'Sid': 'Allow principals in the account to decrypt log files',
                   'Effect': 'Allow',
                   'Principal': {
                     'AWS': '*'
                   },
                   'Action': [
                     'kms:Decrypt',
                     'kms:ReEncryptFrom'
                   ],
                   'Resource': '*',
                   'Condition': {
                     'StringEquals': {
                       'kms:CallerAccount': account_id
                     },
                     'StringLike': {
                       'kms:EncryptionContext:aws:cloudtrail:arn': 'arn:aws:cloudtrail:*:' + account_id + ':trail/*'
                     }
                   }
                 },
                 {
                   'Sid': 'Allow alias creation during setup',
                   'Effect': 'Allow',
                   'Principal': {
                     'AWS': '*'
                   },
                   'Action': 'kms:CreateAlias',
                   'Resource': '*',
                   'Condition': {
                     'StringEquals': {
                       'kms:CallerAccount': account_id,
                       'kms:ViaService': 'ec2.' + bucket_region + '.amazonaws.com'
                     }
                   }
                 },
                 {
                   'Sid': 'Enable cross account log decryption',
                   'Effect': 'Allow',
                   'Principal': {
                     'AWS': '*'
                   },
                   'Action': [
                     'kms:Decrypt',
                     'kms:ReEncryptFrom'
                   ],
                   'Resource': '*',
                   'Condition': {
                     'StringEquals': {
                       'kms:CallerAccount': account_id
                     },
                     'StringLike': {
                       'kms:EncryptionContext:aws:cloudtrail:arn': 'arn:aws:cloudtrail:*:' + account_id + ':trail/*'
                     }
                   }
                 }
               ]
             }

    return Policy




"""
Remediate Evident Signature:

AWS:RDS-011 RDS Minor Upgrades Enabled

Description:

When Amazon Relational Database Service (Amazon RDS) supports a new version of a database engine, you can
upgrade your DB instances to the new version. There are two kinds of upgrades: major version upgrades and
minor version upgrades. Minor upgrades helps maintain a secure and stable RDS with minimal impact on the
application. For this reason, we recommend that your automatic minor upgrade is enabled. Minor version
upgrades only occur automatically if a minor upgrade replaces an unsafe version, such as a minor upgrade
that contains bug fixes for a previous version.

Note: Changing this parameter doesn't result in an outage except in the following case and the change is
asynchronously applied as soon as possible. An outage will result if this parameter is set to true during
the maintenance window, and a newer minor version is available, and RDS has enabled auto patching for that
engine version.

Required Permissions:

- rds:DescribeDBInstances
- rds:ModifyDBInstance

Sample IAM Policy:

{
  "Version": "2012-10-17",
  "Statement": [
    {
      "Sid": "RDSPermissions",
      "Action": [
        "rds:DescribeDBInstances",
        "rds:ModifyDBInstance"
      ],
      "Effect": "Allow",
      "Resource": "*"
    }
  ]
}
"""

import boto3
from botocore.exceptions import ClientError


def remediate(session, alert, lambda_context):
  """
  Main Function invoked by index.py
  """

  instance_id = alert['resource_id']
  region = alert['region']

  rds = session.client('rds', region_name=region)

  try:
    db_instance = rds.describe_db_instances(DBInstanceIdentifier = instance_id)['DBInstances']
  except ClientError as e:
    print(e.response['Error']['Message'])
    return

  try:
    minor_upgrade = db_instance[0]['AutoMinorVersionUpgrade']
  except (KeyError, IndexError):
    pass

  else:

    if minor_upgrade != True:

      try:
        result = rds.modify_db_instance(
          DBInstanceIdentifier = instance_id,
          AutoMinorVersionUpgrade = True
        )
      except ClientError as e:
        print(e.response['Error']['Message'])
      else:
        print('Enabled \'AutoMinorVersionUpgrade\' for RDS instance {}.'.format(instance_id))

  return




import boto3
from importlib import import_module
from io import StringIO
import json
import sys


class ArgumentError(Exception):
    def __init__(self, message):
        self.message = message


class RunbookReturn:
    """The return values from executing a runbook

    """
    def __init__(self, resource_id, region, ret_val):
        self.resource_id = resource_id
        self.region = region
        self.orig = ret_val
        self._lines = ret_val.splitlines()

    @property
    def status(self):
        return self._lines[-1].split(":")[0]

    @property
    def runbook_id(self):
        return self._lines[-1].split(":")[1]

    @property
    def lines(self):
        return self._lines

    def _struct(self):
        return {
            'Status': self.status,
            'Response': self.orig,
            'RunbookId': self.runbook_id,
            'ResourceId': self.resource_id,
            'Region': self.region
        }

    @property
    def json(self):
        return json.dumps(self._struct())

    @property
    def return_value(self):
        return self._struct


class Client:
    """
    Client represents an AWS Client which can call and capture responses from AWS APIs via runbooks which
    contain the actual logic.
    """
    def __init__(self, **kwargs):
        self.args = kwargs

    def get_session(self):
        region = self.args['region']
        if region is None:
            raise ArgumentError("No default region nor command region given")

        access_key = self.args['access_key']
        role_arn = self.args['role_arn']
        role_session_name = self.args['role_session_name']

        if access_key is None:
            if role_arn and role_session_name:
                sts_client = boto3.client('sts')
                sts_response = sts_client.assume_role(
                    RoleArn=role_arn,
                    RoleSessionName=role_session_name,
                    DurationSeconds=self.args['session_duration']
                )
                session = boto3.session.Session(
                    region_name=region,
                    aws_access_key_id=sts_response['Credentials']['AccessKeyId'],
                    aws_secret_access_key=sts_response['Credentials']['SecretAccessKey'],
                    aws_session_token=sts_response['Credentials']['SessionToken']
                )
                return session
            else:
                raise ArgumentError("Must have role_arn and role_session_name when no access_key given")
        else:
            if role_arn and role_session_name:
                sts_client = boto3.client(
                    service_name='sts',
                    aws_access_key_id=access_key,
                    aws_secret_access_key=self.args['secret_key'],
                    verify=self.args['verify_certificate'],
                    config=self.args['config']
                )
                sts_response = sts_client.assume_role(
                    RoleArn=role_arn,
                    RoleSessionName=role_session_name,
                    DurationSeconds=self.args['session_duration']
                )
                session = boto3.session.Session(
                    region_name=region,
                    aws_access_key_id=sts_response['Credentials']['AccessKeyId'],
                    aws_secret_access_key=sts_response['Credentials']['SecretAccessKey'],
                    aws_session_token=sts_response['Credentials']['SessionToken']
                )
                return session
            else:
                session = boto3.session.Session(
                    region_name=region,
                    aws_access_key_id=access_key,
                    aws_secret_access_key=self.args['secret_key'],
                )
                return session

    def run(self, runbook_id, incident):
        session = self.get_session()
        capture = StringIO()
        old_out = sys.stdout
        sys.stdout = capture
        try:
            runbook = import_module('remediate.runbook.' + runbook_id)
            runbook.remediate(session, incident, None)
            print(f"DONE:{runbook_id}")
        except ArgumentError as e:
            print(f'ERROR:Bad Arguments {e.message}')
        except Exception as e:
            print(f'ERROR:{runbook_id} ({incident}) {str(e)}')
        finally:
            sys.stdout = old_out
            return RunbookReturn(incident['resource_id'], incident['region'], capture.getvalue())



"""
Remediate Evident Signature:

AWS:EC2-001 EBS Volume Does Not Have Recent Snapshot

Description:

Elastic block store (EBS) volumes have no snapshots in recent, 15-day history. These are needed to retain
critical system data, security logs, and system state.

EBS volume snapshots are an important tool to record system state, security log data, and critical system
data at various points in your EC2 instance lifecycle. Snapshots provide point-in-time recovery and review
capability that is necessary for many operational and security practices.

Required Permissions:

- ec2:CreateSnapshot
- ec2:DescribeSnapshots

Sample IAM Policy:

{
    "Version": "2012-10-17",
    "Statement": [
        {
            "Sid": "EC2Permissions",
            "Effect": "Allow",
            "Action": [
                "ec2:CreateSnapshot",
                "ec2:DescribeSnapshots"
            ],
            "Resource": [
                "*"
            ]
        }
    ]
}
"""

import boto3
from botocore.exceptions import ClientError
from datetime import datetime
from datetime import date

# Options:
#
# Snapshot age in days
# 
snapshot_age = 15


def remediate(session, alert, lambda_context):
  """
  Main Function invoked by index.py
  """

  volume_id = alert['resource_id']
  region    = alert['region']

  ec2 = session.client('ec2', region_name=region)

  try:
    snapshot = ec2.describe_snapshots(Filters=[{ 'Name': 'volume-id', 'Values': [ volume_id ] }])['Snapshots']
  except ClientError as e:
    print(e.response['Error']['Message'])
    return

  if len(snapshot) > 0:
    snap_date = snapshot[0]['StartTime'].date()    # Snapshot creation date
    today = date.today()                           # Today's date
    delta = today - snap_date                      # The diff

    snapshot_needed = True if (delta.days >= snapshot_age) else False
  else:
    snapshot_needed = True

  if snapshot_needed == True:
    response = new_ebs_snapshot(ec2, volume_id)
    
  return


def new_ebs_snapshot(ec2, volume_id):
  """
  Create a new EBS Volume Snapshot
  """

  try:
    response = ec2.create_snapshot(VolumeId=volume_id, Description='Autoremediate snapshot')
    snapshot_id = response['SnapshotId']

    print('New snapshot {} created for EBS Volume {}.'.format(snapshot_id, volume_id))

  except ClientError as e:
    print(e.response['Error']['Message'])

  return




"""
Remediate Evident Signature:

AWS:CLT-006 S3 Bucket ACL allows public access to CloudTrail logs

Description:

It is recommended that the bucket policy or access control list (ACL) applied to the S3 bucket
that stores CloudTrail logs prevents public access. Allowing public access to CloudTrail log content
may aid an adversary in identifying weaknesses in the affected account's use or configuration.

Required Permissions:

- cloudtrail:DescribeTrails
- s3:GetBucketAcl
- s3:PutBucketAcl

Sample IAM Policy:

{
  "Version": "2012-10-17",
  "Statement": [
    {
      "Sid": "CloudTrailPermissions",
      "Action": [
        "cloudtrail:DescribeTrails"
      ],
      "Effect": "Allow",
      "Resource": "*"
    },
    {
      "Sid": "S3Permissions",
      "Action": [
        "s3:GetBucketAcl",
        "s3:PutBucketAcl"
      ],
      "Effect": "Allow",
      "Resource": "*"
    }
  ]
}
"""

import boto3
from botocore.exceptions import ClientError


def remediate(session, alert, lambda_context):
  """
  Main Function invoked by index.py
  """

  trail_name  = alert['resource_id']
  region      = alert['region']

  clt = session.client('cloudtrail', region_name=region)
  s3  = session.client('s3', region_name=region)

  try:
    trail = clt.describe_trails(trailNameList=[ trail_name ], includeShadowTrails=False)['trailList']
  except ClientError as e:
    print(e.response['Error']['Message'])
    return

  try:
    bucket_name = trail[0]['S3BucketName']
  except (IndexError, KeyError):
    print('Could not find S3 bucket for Trail {}.'.format(trail_name))
    return

  try:
    bucket_acl = s3.get_bucket_acl(Bucket=bucket_name)
  except ClientError as e:
    print(e.response['Error']['Message'])
    return

  new_bucket_acl = {}
  new_bucket_acl['Owner'] = bucket_acl['Owner']

  new_grants = []
  public = False
  
  for grant in bucket_acl['Grants']:
    try:
      grant_type = grant['Grantee']['Type']
      grant_uri  = grant['Grantee']['URI']
    except KeyError:
      new_grants.append(grant)
      continue

    if ((grant_type == 'Group' and 'AllUsers' in grant_uri) or
        (grant_type == 'Group' and 'AuthenticatedUsers' in grant_uri)):
      public = True
    else:
      new_grants.append(grant)

  new_bucket_acl['Grants'] = new_grants

  # Remediate
  if public == True:
    result = remove_public_acl(s3, bucket_name, new_bucket_acl, trail_name)
    
  return


def remove_public_acl(s3, bucket_name, new_bucket_acl, trail_name):
  """
  Remove S3 Bucket Public ACL Policy
  """

  try:
    result = s3.put_bucket_acl(
      AccessControlPolicy = new_bucket_acl,
      Bucket = bucket_name
    )
  except ClientError as e:
    print(e.response['Error']['Message'])
  else:
    print('S3 bucket {} ACL policy, removed public access for Trail {}.'.format(bucket_name, trail_name))

  return




"""
Remediate Evident Signature:

AWS:EC2-042 EBS Snapshot set with Public Permissions

Description:

To avoid exposing personal and sensitive data, we recommend against sharing your EBS snapshots with
all AWS accounts.

Required Permissions:

- ec2:DescribeSnapshotAttribute
- ec2:ModifySnapshotAttribute

Sample IAM Policy:

{
  "Version": "2012-10-17",
  "Statement": [
    {
      "Sid": "EC2Permissions",
      "Action": [
        "ec2:DescribeSnapshotAttribute",
        "ec2:ModifySnapshotAttribute"
      ],
      "Effect": "Allow",
      "Resource": "*"
    }
  ]
}
"""

import boto3
from botocore.exceptions import ClientError


def remediate(session, alert, lambda_context):
  """
  Main Function invoked by index.py
  """

  snapshot_id = alert['resource_id']
  region = alert['region']

  ec2 = session.client('ec2', region_name=region)

  try:
    snap_attrib = ec2.describe_snapshot_attribute(
    Attribute = 'createVolumePermission',
    SnapshotId = snapshot_id
    )
  except ClientError as e:
    print(e.response['Error']['Message'])
    return

  vol_perms = snap_attrib['CreateVolumePermissions'] if ('CreateVolumePermissions' in snap_attrib) else ''

  public = False

  for perm in vol_perms:
    try:
      if perm['Group'] == 'all':
        public = True
    except KeyError:
      continue

  if public == True:
    result = remove_pub_snapshot_attrib(ec2, snapshot_id) 

  return


def remove_pub_snapshot_attrib(ec2, snapshot_id):
  """
  Remove Public Snaphot Attribute
  """

  try:
    result = ec2.modify_snapshot_attribute(
      Attribute = 'createVolumePermission',
      GroupNames = [
        'all',
      ],
      OperationType = 'remove',
      SnapshotId = snapshot_id
    )
  except ClientError as e:
    print(e.response['Error']['Message'])
  else:
    print('Removed "Public" attribute from EBS snapshot {}.'.format(snapshot_id))

  return




"""
Remediate Evident Signature:

AWS:CFM-003 CFN Stack Termination Protection

Description:

You can prevent a stack from being accidentally deleted by enabling termination protection on the stack.
If a user attempts to delete a stack with termination protection enabled, the deletion fails and the stack
including its status remains unchanged.

Required Permissions:

- cloudformation:DescribeStacks
- cloudformation:UpdateTerminationProtection

Sample IAM Policy:

{
  "Version": "2012-10-17",
  "Statement": [
    {
      "Sid": "CloudFormationPermissions",
      "Action": [
        "cloudformation:DescribeStacks",
        "cloudformation:UpdateTerminationProtection"
      ],
      "Effect": "Allow",
      "Resource": "*"
    }
  ]
}
"""

import boto3
from botocore.exceptions import ClientError


def remediate(session, alert, lambda_context):
  """
  Main Function invoked by index.py
  """

  stack_id = alert['resource_id']
  region   = alert['region']

  cfn = session.client('cloudformation', region_name=region)

  try:
    stack = cfn.describe_stacks(StackName=stack_id)['Stacks']
  except ClientError as e:
    print(e.response['Error']['Message'])
    return

  stack_name = stack[0]['StackName']

  # Don't check stack with a status of..
  if stack[0]['StackStatus'] in ("DELETE_COMPLETE", "DELETE_IN_PROGRESS", "DELETE_FAILED"):
    return

  if stack[0]['EnableTerminationProtection'] != True:
    result = enable_term_protection(cfn, stack_name)

  return


def enable_term_protection(cfn, stack_name):
  """
  Enable CloudFormation Termination Protection
  """

  try:
    result = cfn.update_termination_protection(StackName=stack_name, EnableTerminationProtection=True)
  except ClientError as e:
    print(e.response['Error']['Message'])
  else:
    print('Enabled termination protection for CloudFormation Stack {}.'.format(stack_name))

  return




"""
Remediate Custom Evident Signature:

AWS:VPC-Default AWS Default VPC Check

https://github.com/PaloAltoNetworks/evident-custom-signatures/blob/master/AWS/sample_codes/AWS_EC2%20-%20default_vpc_check.rb

Description:

Delete the AWS default VPC from offending regions.

Required Permissions:

- ec2:DeleteInternetGateway
- ec2:DeleteNetworkAcl
- ec2:DeleteRouteTable
- ec2:DeleteSecurityGroup
- ec2:DeleteSubnet
- ec2:DeleteVpc
- ec2:DescribeInternetGateways
- ec2:DescribeNetworkAcls
- ec2:DescribeNetworkInterfaces
- ec2:DescribeRouteTables
- ec2:DescribeSecurityGroups
- ec2:DescribeSubnets
- ec2:DescribeVpcs
- ec2:DetachInternetGateway

Sample IAM Policy:

{
    "Version": "2012-10-17",
    "Statement": [
        {
            "Sid": "EC2Permissions",
            "Effect": "Allow",
            "Action": [
                "ec2:DeleteInternetGateway",
                "ec2:DeleteNetworkAcl",
                "ec2:DeleteRouteTable",
                "ec2:DeleteSecurityGroup",
                "ec2:DeleteSubnet",
                "ec2:DeleteVpc",
                "ec2:DescribeInternetGateways",
                "ec2:DescribeNetworkAcls",
                "ec2:DescribeNetworkInterfaces",
                "ec2:DescribeRouteTables",
                "ec2:DescribeSecurityGroups",
                "ec2:DescribeSubnets",
                "ec2:DescribeVpcs",
                "ec2:DetachInternetGateway"
            ],
            "Resource": [
                "*"
            ]
        }
    ]
}
"""

import boto3
from botocore.exceptions import ClientError


def remediate(session, alert, lambda_context):
  """
  Main Function invoked by index.py
  """

  vpc_id = alert['resource_id']
  region = alert['region']

  ec2  = session.client('ec2', region_name=region)

  try:
    vpc = ec2.describe_vpcs(VpcIds=[ vpc_id ])
  except ClientError as e:
    print(e.response['Error']['Message'])
    return

  else:
    is_default = vpc['Vpcs'][0]['IsDefault']

  # Is this the default VPC?
  if is_default != True:
    print('VPC {} is not the default.'.format(vpc_id))
    return

  # Are there any existing resources?  Since most resources attach an ENI, let's check..
  try:
    eni = ec2.describe_network_interfaces(
            Filters = [
              {
                'Name': 'vpc-id',
                'Values': [ vpc_id ]
              }
            ]
          )['NetworkInterfaces']
  except ClientError as e:
    print(e.response['Error']['Message'])
    return

  if eni:
    print('VPC {} has existing resources.'.format(vpc_id))
    return

  # Do the work..
  result = delete_igw(ec2, vpc_id)
  result = delete_subs(ec2, vpc_id)
  result = delete_rtbs(ec2, vpc_id)
  result = delete_acls(ec2, vpc_id)
  result = delete_sgps(ec2, vpc_id)
  result = delete_vpc(ec2, vpc_id)

  return


def delete_igw(ec2, vpc_id):
  """
  Detach and delete the internet gateway
  """

  try:
    igw = ec2.describe_internet_gateways(
            Filters = [
              {
                'Name' : 'attachment.vpc-id',
                'Values' : [ vpc_id ]
              }
            ]
          )['InternetGateways']
  except ClientError as e:
    print(e.response['Error']['Message'])

  if igw:
    igw_id = igw[0]['InternetGatewayId']

    try:
      ec2.detach_internet_gateway(InternetGatewayId=igw_id, VpcId=vpc_id)
    except ClientError as e:
      print(e.response['Error']['Message'])

    try:
      ec2.delete_internet_gateway(InternetGatewayId=igw_id)
    except ClientError as e:
      print(e.response['Error']['Message'])

  return


def delete_subs(ec2, vpc_id):
  """
  Delete the subnets
  """

  try:
    subs = ec2.describe_subnets(
             Filters = [
               {
                 'Name' : 'vpc-id',
                 'Values' : [ vpc_id ]
               }
             ]
           )['Subnets']
  except ClientError as e:
    print(e.response['Error']['Message'])

  if subs:
    for sub in subs:
      sub_id = sub['SubnetId']

      try:
        ec2.delete_subnet(SubnetId=sub_id)
      except ClientError as e:
        print(e.response['Error']['Message'])

  return


def delete_rtbs(ec2, vpc_id):
  """
  Delete the route tables
  """

  try:
    rtbs = ec2.describe_route_tables(
             Filters = [
               {
                 'Name' : 'vpc-id',
                 'Values' : [ vpc_id ]
               }
             ]
           )['RouteTables']
  except ClientError as e:
    print(e.response['Error']['Message'])

  if rtbs:
    for rtb in rtbs:
      main = 'false'
      for assoc in rtb['Associations']:
        main = assoc['Main']
      if main == True:
        continue
      rtb_id = rtb['RouteTableId']
        
      try:
        ec2.delete_route_table(RouteTableId=rtb_id)
      except ClientError as e:
        print(e.response['Error']['Message'])

  return


def delete_acls(ec2, vpc_id):
  """
  Delete the network access lists (NACLs)
  """

  try:
    acls = ec2.describe_network_acls(
             Filters = [
               {
                 'Name' : 'vpc-id',
                 'Values' : [ vpc_id ]
               }
             ]
           )['NetworkAcls']
  except ClientError as e:
    print(e.response['Error']['Message'])

  if acls:
    for acl in acls:
      default = acl['IsDefault']
      if default == True:
        continue
      acl_id = acl['NetworkAclId']

      try:
        ec2.delete_network_acl(NetworkAclId=acl_id)
      except ClientError as e:
        print(e.response['Error']['Message'])

  return


def delete_sgps(ec2, vpc_id):
  """
  Delete any security groups
  """

  try:
    sgps = ec2.describe_security_groups(
             Filters = [
               {
                 'Name' : 'vpc-id',
                 'Values' : [ vpc_id ]
               }
             ]
           )['SecurityGroups']
  except ClientError as e:
    print(e.response['Error']['Message'])

  if sgps:
    for sgp in sgps:
      default = sgp['GroupName']
      if default == 'default':
        continue
      sg_id = sgp['GroupId']

      try:
        ec2.delete_security_group(GroupId=sg_id)
      except ClientError as e:
        print(e.response['Error']['Message'])

  return


def delete_vpc(ec2, vpc_id):
  """
  Delete the VPC
  """

  try:
    ec2.delete_vpc(VpcId=vpc_id)
  except ClientError as e:
    print(e.response['Error']['Message'])

  else:
    print('VPC {} has been deleted.'.format(vpc_id))

  return




"""
Remediate Evident Signature:

AWS:KMS-002: KMS Key Scheduled for Deletion

Description:

The recommended action to remedy this problem is to cancel this deletion and disable the key instead.

The minimum amount of time required for scheduling a key deletion is seven days allowing for a pending deletion operation to be canceled before the deletion date.

Required Permissions:

- kms:DescribeKeys
- kms:DisableKey
- kms:CancelKeyDeletion

Sample IAM Policy:

{
  "Version": "2012-10-17",
  "Statement": [
    {
      "Sid": "KMSPermissions",
      "Action": [
	"kms:DescribeKeys",
        "kms:DisableKey",
        "kms:CancelKeyDeletion"
      ],
      "Effect": "Allow",
      "Resource": "*"
    }
  ]
}
"""

import boto3
from botocore.exceptions import ClientError


def remediate(session, alert, lambda_context):
  """
  Main Function invoked by index.py
  """

  key_id = alert['resource_id']
  region = alert['region']

  kms = session.client('kms', region_name=region)

  try:
    key_metadata = kms.describe_key(KeyId = key_id)['KeyMetadata']
  
  except ClientError as e:
    print(e.response['Error']['Message'])
    return

  key_status = key_metadata['KeyState']
  
  if key_status == "PendingDeletion":
    try:
      result = kms.cancel_key_deletion(KeyId = key_id)
 
    except ClientError as e:
      print(e.response['Error']['Message'])
      return

    try:
      result = kms.disable_key(KeyId = key_id)
    except ClientError as e:
      print(e.response['Error']['Message'])
    else:
      print('KMS key disabled for Customer Master Key: {}.'.format(key_id))
  return



"""
Remediate Evident Signature:

AWS:SSS-001 S3 Object Versioning Not Enabled

Description:

S3 Object Versioning is an important capability in protecting your data within a bucket. Once you enable
Object Versioning, you cannot remove it; you can suspend Object Versioning at any time on a bucket if you
do not wish for it to persist.

Required Permissions:

- s3:PutBucketVersioning

Sample IAM Policy:

{
  "Version": "2012-10-17",
  "Statement": [
    {
      "Sid": "S3Permissions",
      "Action": [
        "s3:PutBucketVersioning"
      ],
      "Effect": "Allow",
      "Resource": "*"
    }
  ]
}
"""

import boto3
from botocore.exceptions import ClientError


def remediate(session, alert, lambda_context):
  """
  Main Function invoked by index.py
  """

  bucket = alert['resource_id']
  region = alert['region']

  s3 = session.client('s3', region_name=region)

  try:
    result = s3.put_bucket_versioning(
      Bucket = bucket,
      VersioningConfiguration = {
        'Status': 'Enabled'
      }
    )
  except ClientError as e:
    print(e.response['Error']['Message'])
    return

  else:
    print('Object Versioning enabled for S3 bucket: {}.'.format(bucket))

  return




"""
Remediate Evident Signature:

AWS:SSS-008 S3 Bucket has Global ACL Permissions enabled

Description:

Remove the existence of Global ACL permissions on S3 Bucket for the All Users and Authenticated Users groups.

Required Permissions:

- s3:GetBucketAcl
- s3:PutBucketAcl

Sample IAM Policy:

{
  "Version": "2012-10-17",
  "Statement": [
    {
      "Sid": "S3Permissions",
      "Action": [
        "s3:GetBucketAcl",
        "s3:PutBucketAcl"
      ],
      "Effect": "Allow",
      "Resource": "*"
    }
  ]
}
"""

import boto3
from botocore.exceptions import ClientError


def remediate(session, alert, lambda_context):
  """
  Main Function invoked by index.py
  """

  bucket_name  = alert['resource_id']
  region       = alert['region']

  s3 = session.client('s3', region_name=region)

  try:
    bucket_acl = s3.get_bucket_acl(Bucket=bucket_name)
  except ClientError as e:
    print(e.response['Error']['Message'])
    return

  new_bucket_acl = {}
  new_bucket_acl['Owner'] = bucket_acl['Owner']

  new_grants = []
  public = False
  
  for grant in bucket_acl['Grants']:
    try:
      grant_type = grant['Grantee']['Type']
      grant_uri  = grant['Grantee']['URI']
    except KeyError:
      new_grants.append(grant)
      continue

    if ((grant_type == 'Group' and 'AllUsers' in grant_uri) or
        (grant_type == 'Group' and 'AuthenticatedUsers' in grant_uri)):
      public = True
    else:
      new_grants.append(grant)

  new_bucket_acl['Grants'] = new_grants

  # Remediate
  if public == True:
    result = remove_public_acl(s3, bucket_name, new_bucket_acl)
    
  return


def remove_public_acl(s3, bucket_name, new_bucket_acl):
  """
  Remove S3 Bucket Global ACL Policy
  """

  try:
    result = s3.put_bucket_acl(
      AccessControlPolicy = new_bucket_acl,
      Bucket = bucket_name
    )
  except ClientError as e:
    print(e.response['Error']['Message'])
  else:
    print('Global access removed from S3 bucket {} ACL policy.'.format(bucket_name))

  return




"""
Remediate Evident Signature:

AWS:ELB-015 Application Load Balancer (elbv2) Access Log Enabled

Description:

Ensure that your AWS Application Elastic Load Balancers has access logging to analyze traffic patterns
and identify and troubleshoot security issues.

Required Permissions:

- elasticloadbalancing:DescribeLoadBalancerAttributes
- elasticloadbalancing:DescribeLoadBalancers
- elasticloadbalancing:ModifyLoadBalancerAttributes
- s3:CreateBucket
- s3:PutBucketPolicy
- s3:PutObject

Sample IAM Policy:

{
  "Version": "2012-10-17",
  "Statement": [
    {
      "Sid": "ELBPermissions",
      "Action": [
        "elasticloadbalancing:DescribeLoadBalancerAttributes",
        "elasticloadbalancing:DescribeLoadBalancers",
        "elasticloadbalancing:ModifyLoadBalancerAttributes"
      ],
      "Effect": "Allow",
      "Resource": "*"
    },
    {
      "Sid": "S3Permissions",
      "Action": [
        "s3:CreateBucket",
        "s3:PutBucketPolicy",
        "s3:PutObject"
      ],
      "Effect": "Allow",
      "Resource": "*"
    }
  ]
}
"""

import json
import boto3
from botocore.exceptions import ClientError


def remediate(session, alert, lambda_context):
  """
  Main Function invoked by index.py
  """

  elb_name = alert['resource_id']
  region   = alert['region']

  elbv2 = session.client('elbv2', region_name=region)
  s3 = session.client('s3', region_name=region)

  try:
    elb = elbv2.describe_load_balancers(Names=[ elb_name ])['LoadBalancers']
  except ClientError as e:
    print(e.response['Error']['Message'])
    return

  try:
    elb_arn = elb[0]['LoadBalancerArn']
  except (KeyError, IndexError):
    print('Error: Unable to find the ARN for Application ELB {}.'.format(elb_name))
    return

  else:
    account_id = elb_arn.split(':')[4]

  try:
    attribs = elbv2.describe_load_balancer_attributes(LoadBalancerArn=elb_arn)['Attributes']
  except ClientError as e:
    print(e.response['Error']['Message'])
    return

  logging = 'none'

  for attrib in attribs:
    if attrib['Key'] == 'access_logs.s3.enabled':
      logging = attrib['Value']

  if logging != 'true':

    bucket_name = new_s3_bucket(s3, elb_name, account_id, region)

    if bucket_name != 'fail':
      result = enable_access_log(elbv2, elb_arn, elb_name, bucket_name, region)

  return


def enable_access_log(elbv2, elb_arn, elb_name, bucket_name, region):
  """
  Enable ELB Application (elbv2) Access Log
  """

  try:
    result = elbv2.modify_load_balancer_attributes(
      LoadBalancerArn = elb_arn,
      Attributes = [
        {
          'Key': 'access_logs.s3.enabled',
          'Value': 'true'
        },
        {
          'Key': 'access_logs.s3.bucket',
          'Value': bucket_name
        },
        {
          'Key': 'access_logs.s3.prefix',
          'Value': elb_name
        }
      ]
    )
  except ClientError as e:
    if 'Access Denied' in e.response['Error']['Message']:
      print('Access Denied: Check the AWS ELB Account Id for the {} region.'.format(region))
    else:
      print(e.response['Error']['Message'])

  else:
    print('Enabled Access Log for Application ELB {}.'.format(elb_name))

  return


def new_s3_bucket(s3, elb_name, account_id, region):
  """
  Create new S3 Bucket
  """

  bucket_name = 'elbv2logs-' + account_id + '-' + region

  try:
    if region == 'us-east-1':
      result = s3.create_bucket(
        ACL = 'private',
        Bucket = bucket_name
      )
    else:
      result = s3.create_bucket(
        ACL = 'private',
        Bucket = bucket_name,
        CreateBucketConfiguration = {'LocationConstraint': region}
      )

    print('New S3 bucket created: {}'.format(bucket_name))

  except ClientError as e:
    if e.response['Error']['Code'] == 'BucketAlreadyExists':
      print('Using existing S3 bucket: {}'.format(bucket_name))
    elif e.response['Error']['Code'] == 'BucketAlreadyOwnedByYou':
      print('Using already owned and existing S3 bucket: {}'.format(bucket_name))
    else:
      print(e.response['Error']['Message'])
      return 'fail'

  # Create ELB folder/ prefix
  try:
    result = s3.put_object(
      Bucket = bucket_name,
      Key = elb_name +'/'
    )
  except ClientError as e:
    print(e.response['Error']['Message'])
    return 'fail'

  # Create ELB logging policy
  try:
    result = s3.put_bucket_policy(
      Bucket = bucket_name,
      Policy = json.dumps(BucketTemplate.BucketPolicy(bucket_name, account_id, region))
    )
  except ClientError as e:
    if 'Invalid principal' in e.response['Error']['Message']:
      print('Invalid principal: Check the AWS ELB Account Id for the {} region.'.format(region))
    else:
      print(e.response['Error']['Message'])
    return 'fail'

  return bucket_name


class BucketTemplate():

  def BucketPolicy(bucket_name, account_id, region):

    elb_account_id = '123456789012'

    # Reference:
    # https://docs.aws.amazon.com/elasticloadbalancing/latest/classic/enable-access-logs.html

    if region == 'us-east-1':      elb_account_id = '127311923021'
    if region == 'us-east-2':      elb_account_id = '033677994240'
    if region == 'us-west-1':      elb_account_id = '027434742980'
    if region == 'us-west-2':      elb_account_id = '797873946194'
    if region == 'ca-central-1':   elb_account_id = '985666609251'
    if region == 'eu-central-1':   elb_account_id = '054676820928'
    if region == 'eu-west-1':      elb_account_id = '156460612806'
    if region == 'eu-west-2':      elb_account_id = '652711504416'
    if region == 'eu-west-3':      elb_account_id = '009996457667'
    if region == 'ap-northeast-1': elb_account_id = '582318560864'
    if region == 'ap-northeast-2': elb_account_id = '600734575887'
    if region == 'ap-northeast-3': elb_account_id = '383597477331'
    if region == 'ap-southeast-1': elb_account_id = '114774131450'
    if region == 'ap-southeast-2': elb_account_id = '783225319266'
    if region == 'ap-south-1':     elb_account_id = '718504428378'
    if region == 'sa-east-1':      elb_account_id = '507241528517'

    Policy = {
               'Version': '2012-10-17',
               'Statement': [
                 {
                   'Sid': 'ELBLoggingPolicy',
                   'Effect': 'Allow',
                   'Principal': {
                     'AWS': 'arn:aws:iam::' + elb_account_id + ':root'
                   },
                   'Action': 's3:PutObject',
                   'Resource': 'arn:aws:s3:::' + bucket_name + '/*' + '/AWSLogs/' + account_id + '/*'
                 }
               ]
             }

    return Policy




"""
Remediate Evident Signature:

AWS:EC2-038 Default VPC Security Group Allows Traffic from All Protocols and Ports

Description:

The default security group within a VPC is not configured to restrict outbound traffic and limits
inbound traffic to resources within the same group.

While the default security group cannot be deleted, it can be configured in a way to mitigate risk
and increase security.

**Note: Remediation revokes all ingress and egress rules from the default security group in order to
        comply with CIS Benchmark 4.3.

Required Permissions:

- ec2:DescribeSecurityGroups
- ec2:RevokeSecurityGroupIngress
- ec2:RevokeSecurityGroupEgress

Sample IAM Policy:

{
    "Version": "2012-10-17",
    "Statement": [
        {
            "Sid": "EC2Permissions",
            "Effect": "Allow",
            "Action": [
                "ec2:DescribeSecurityGroups",
                "ec2:RevokeSecurityGroupIngress",
                "ec2:RevokeSecurityGroupEgress"
            ],
            "Resource": [
                "*"
            ]
        }
    ]
}
"""

import boto3
from botocore.exceptions import ClientError


def remediate(session, alert, lambda_context):
  """
  Main Function invoked by index.py
  """

  sg_id  = alert['resource_id']
  region = alert['region']

  ec2 = session.client('ec2', region_name=region)

  try:
    group = ec2.describe_security_groups(GroupIds=[ sg_id, ])['SecurityGroups']
  except ClientError as e:
    print(e.response['Error']['Message'])
    return

  # Check for default security group
  if group[0]['GroupName'] != 'default':
    print('Security group {} is not the default.'.format(sg_id))
    return

  # Revoke all ingress permissions
  try:
    ingress_perms = group[0]['IpPermissions']
  except (IndexError, KeyError):
    ingress_perms = ''

  for ip_perm in ingress_perms:
    remove_sg_rule(ec2, sg_id, ip_perm, revoke='ingress')

  # Revoke all egress permissions
  try:
    egress_perms = group[0]['IpPermissionsEgress']
  except (IndexError, KeyError):
    egress_perms = ''

  for ip_perm in egress_perms:
    remove_sg_rule(ec2, sg_id, ip_perm, revoke='egress')

  return


def remove_sg_rule(ec2, sg_id, ip_perm, revoke):
  """
  Revoke Security Group Rules
  """

  revoke_args = {
    'GroupId' :  sg_id,
    'IpPermissions' : [ ip_perm ]
  }

  if revoke == 'ingress':
    try:
      ec2.revoke_security_group_ingress(**revoke_args)
    except ClientError as e:
      print(e.response['Error']['Message'])
      return

  elif revoke == 'egress':
    try:
      ec2.revoke_security_group_egress(**revoke_args)
    except ClientError as e:
      print(e.response['Error']['Message'])
      return
  else:
    return

  print('Revoked {} rule {} from default security group {}.'.format(revoke, ip_perm, sg_id))

  return




"""
Remediate Evident Signature:

AWS:VPC-013 VPC Elastic IP Limit

Description:

If your account is approaching the Elastic IP limitation per region, this remediation script will attempt
to release any/all unassociated EIPs.

Required Permissions:

- ec2:DescribeAddresses
- ec2:ReleaseAddress

Sample IAM Policy:

{
  "Version": "2012-10-17",
  "Statement": [
    {
      "Sid": "EC2Permissions",
      "Action": [
        "ec2:DescribeAddresses",
        "ec2:ReleaseAddress"
      ],
      "Effect": "Allow",
      "Resource": "*"
    }
  ]
}
"""

import boto3
from botocore.exceptions import ClientError


def remediate(session, alert, lambda_context):
  """
  Main Function invoked by index.py
  """

  resource = None
  region   = alert['region']

  ec2 = session.client('ec2', region_name=region)

  try:
    eips = ec2.describe_addresses()['Addresses']
  except ClientError as e:
    print(e.response['Error']['Message'])
    return

  unassociated = []

  for eip in eips:
    if 'AssociationId' not in eip:
      unassociated.append(eip['AllocationId'])

  for id in unassociated:
    try:
      result = ec2.release_address(AllocationId=id)
    except ClientError as e:
      print(e.response['Error']['Message'])
    else:
      print('Released unassociated EIP {} in the {} region.'.format(id, region))

  return




"""
Remediate Evident Signature:

AWS:ELB-009 ELB (Classic) Connection Draining

Description:

Enable connection draining, and then specify a maximum time for the Load Balancer to keep connections alive
before reporting the instance as de-registered. The maximum timeout value can be set between 1 and 3,600
seconds (the default is 300 seconds). When the maximum time limit is reached, the load balancer forcibly
closes connections to the de-registering instance.

If an instance becomes unhealthy, the load balancer reports the instance state as OutOfService. If there are
in-flight requests made to the unhealthy instance, they are completed.

Required Permissions:

- elasticloadbalancing:DescribeLoadBalancerAttributes
- elasticloadbalancing:ModifyLoadBalancerAttributes

Sample IAM Policy:

{
  "Version": "2012-10-17",
  "Statement": [
    {
      "Sid": "ELBPermissions",
      "Action": [
        "elasticloadbalancing:DescribeLoadBalancerAttributes",
        "elasticloadbalancing:ModifyLoadBalancerAttributes"
      ],
      "Effect": "Allow",
      "Resource": "*"
    }
  ]
}
"""

import boto3
from botocore.exceptions import ClientError


def remediate(session, alert, lambda_context):
  """
  Main Function invoked by index.py
  """

  elb_name = alert['resource_id']
  region   = alert['region']

  elb = session.client('elb', region_name=region)

  try:
    attribs = elb.describe_load_balancer_attributes(LoadBalancerName=elb_name)['LoadBalancerAttributes']
  except ClientError as e:
    print(e.response['Error']['Message'])
    return

  draining = attribs['ConnectionDraining']

  if draining['Enabled'] != True:
    result = enable_conn_draining(elb, elb_name)

  return


def enable_conn_draining(elb, elb_name):
  """
  Enable ELB (Classic) Connection Draining
  """

  try:
    result = elb.modify_load_balancer_attributes(
      LoadBalancerName = elb_name,
      LoadBalancerAttributes = {
        'ConnectionDraining': {
          'Enabled': True,
          'Timeout': 300
        }
      }
    )
  except ClientError as e:
    print(e.response['Error']['Message'])
  else:
    print('Enabled Connection Draining for ELB {}.'.format(elb_name))

  return




"""
AWS:IAM-002 Password Policy

For a pass alert, the account need to have at least the following password policy:
- MinimumPasswordLength: 14 or more
- RequireLowercaseCharacters: True
- RequireUppercaseCharacters: True
- RequireNumbers: True
- RequireNumbers: True
- MaxPasswordAge: 90 or less
- AllowUsersToChangePassword: True
- PasswordReusePrevention: 1 or more

If there's an existing password policy, the remediation script will:
- take the higher number for MinimumPasswordLength and PasswordReusePrevention
- take the lower number for MaxPasswordAge

Required permissions / Sample IAM Policy:
{
  "Version": "2012-10-17",
  "Statement": [
    {
      "Sid": "Stmt1507759700000",
      "Effect": "Allow",
      "Action": [
        "iam:UpdateAccountPasswordPolicy"
      ],
      "Resource": [
        "*"
      ]
    }
  ]
}

"""

# If set to True, no change will be made to password policy.
# If set to False, password policy will be changed
dry_run = True

def remediate(session, alert, lambda_context):
    enforced_policy = {
        'MinimumPasswordLength': 14,
        'MaxPasswordAge': 90,
        'RequireLowercaseCharacters': True,
        'RequireUppercaseCharacters': True,
        'RequireNumbers': True,
        'RequireSymbols': True,
        'AllowUsersToChangePassword': True,
        'PasswordReusePrevention': 1
    }

    # If there is an existing policy, grab the more restrictive MinimumPasswordLength, PasswordReusePrevention, and MaxPasswordAge
    if 'passwordPolicy' in alert['metadata']:
        print("current password policy", alert['metadata']['passwordPolicy'])
        current_policy = {k.lower(): v for k, v in alert['metadata']['passwordPolicy'].items()}

        for key in ['MinimumPasswordLength','PasswordReusePrevention']:
            if key.lower() in current_policy and current_policy[key.lower()] is not None:
                enforced_policy[key] = max(current_policy[key.lower()],enforced_policy[key])
        
        for key in ['MaxPasswordAge']:
            if key.lower() in current_policy and current_policy[key.lower()] is not None:
                enforced_policy[key] = min(current_policy[key.lower()],enforced_policy[key])
    else:
        current_policy = {}


    iam_client = session.client('iam')
    try:
        print("current password policy: ", current_policy)
        print('Updating account password policy with: ', enforced_policy)

        if dry_run:
            print('Dry run mode. No changes made')
            return
        else:
            resp = iam_client.update_account_password_policy(**enforced_policy)

    except Exception as e:
        raise e
    
    print('Password policy updated')
    return 



"""
Remediate Evident Signature:

AWS:CLT-004 CloudTrail logs not integrated with CloudWatch

Description:

CloudTrail logs are not automatically sent to CloudWatch Logs. CloudWatch integration facilitates real-time
and historic activity logging based on user, API, resource, and IP address. It will also establish alarms and
notifications for anomalous or sensitive account activity.

Required Permissions:

- iam:CreateRole
- iam:GetRole
- iam:PutRolePolicy
- logs:CreateLogGroup
- logs:DescribeLogGroups
- logs:PutRetentionPolicy
- cloudtrail:UpdateTrail

Sample IAM Policy:

{
  "Version": "2012-10-17",
  "Statement": [
    {
      "Sid": "IAMPermissions",
      "Action": [
        "iam:CreateRole",
        "iam:GetRole",
        "iam:PutRolePolicy"
      ],
      "Effect": "Allow",
      "Resource": "*"
    },
    {
      "Sid": "CloudWatchLogsPermissions",
      "Action": [
        "logs:CreateLogGroup",
        "logs:DescribeLogGroups",
        "logs:PutRetentionPolicy"
      ],
      "Effect": "Allow",
      "Resource": "*"
    },
    {
      "Sid": "CloudTrailPermissions",
      "Action": [
        "cloudtrail:UpdateTrail"
      ],
      "Effect": "Allow",
      "Resource": "*"
    }
  ]
}
"""

import json
import boto3
from botocore.exceptions import ClientError
from time import sleep

# Options:
#
# AWS IAM Role/ Policy and CloudWatch Logs Group Name
#
role_name   = 'CloudTrail_CloudWatchLogs_Role'
policy_name = 'CWLogsAllowDeliveryPolicy'
log_group_name = 'CloudTrail/DefaultLogGroup'


def remediate(session, alert, lambda_context):
  """
  Main Function invoked by index.py
  """

  trail_name = alert['resource_id']
  region     = alert['region']

  iam  = session.client('iam', region_name=region)
  logs = session.client('logs', region_name=region)
  clt  = session.client('cloudtrail', region_name=region)

  # Create IAM Role and Policy
  role_arn = new_iam_role(iam)

  # Create CloudWatch Logs group
  log_group_arn = new_log_group(logs) if (role_arn != 'fail') else 'fail'

  # Update Trail
  trail_status = update_trail(clt, trail_name, log_group_arn, role_arn) if (log_group_arn != 'fail') else 'fail'

  # Result
  if trail_status != 'fail':
    print('Integrated Trail {} with CloudWatch Logs group {}.'.format(trail_name, log_group_name))
  else:
    print('Failed to integrated Trail {} with CloudWatch Logs.'.format(trail_name))

  return


def new_iam_role(iam):
  """
  Create new IAM Role and Inline Policy
  """

  try:
    role = iam.create_role(
      Path = '/',
      RoleName = role_name,
      AssumeRolePolicyDocument = json.dumps(Templates.RolePolicy)
    )
    role_arn  = role['Role']['Arn']

    print('New IAM Role created: {}'.format(role_arn))

  except ClientError as e:
    if e.response['Error']['Code'] == 'EntityAlreadyExists':
      role = iam.get_role(
        RoleName = role_name
      )
      role_arn  = role['Role']['Arn']

      print('Using existing IAM Role: {}'.format(role_arn))

    else:
      print(e.response['Error']['Message'])
      return 'fail'

  try:
    policy = iam.put_role_policy(
      RoleName = role_name,
      PolicyName = policy_name,
      PolicyDocument = json.dumps(Templates.LogsPolicy)
    )
  except ClientError as e:
    print(e.response['Error']['Message'])
    return 'fail'

  sleep(10)  # Wait for IAM resource to be available. >> Gotta be a better way.. { wait? get? }.

  return role_arn


def new_log_group(logs):
  """
  Create CloudWatch Logs group
  """

  try:
    group = logs.create_log_group(
      logGroupName = log_group_name
    )

    print('New CloudWatch Logs group created: {}'.format(log_group_name))

  except ClientError as e:
    if e.response['Error']['Code'] == 'ResourceAlreadyExistsException':
      print('Using existing CloudWatch Log group: {}'.format(log_group_name))
    else:
      print(e.response['Error']['Message'])
      return 'fail'

  try:
    retention = logs.put_retention_policy(
      logGroupName = log_group_name,
      retentionInDays = 30
    )
  except ClientError as e:
    print(e.response['Error']['Message'])

  try:
    log_group = logs.describe_log_groups(logGroupNamePrefix = log_group_name)['logGroups']
  except ClientError as e:
    print(e.response['Error']['Message'])
    return 'fail'

  else:
    log_group_arn = log_group[0]['arn']

  return log_group_arn
  

def update_trail(clt, trail_name, log_group_arn, role_arn):
  """
  Update Trail to integrate with CloudWatch Logs
  """

  try:
    result = clt.update_trail(
      Name = trail_name,
      CloudWatchLogsLogGroupArn = log_group_arn,
      CloudWatchLogsRoleArn = role_arn,
    )
  except ClientError as e:
    print(e.response['Error']['Message'])
    return 'fail'

  return


class Templates:

  RolePolicy =  {
                  'Version': '2012-10-17',
                  'Statement': [
                    {
                      'Sid': '',
                      'Effect': 'Allow',
                      'Principal': {
                        'Service': 'cloudtrail.amazonaws.com'
                      },
                      'Action': 'sts:AssumeRole'
                    }
                  ]
                }

  LogsPolicy =  {
                  'Version': '2012-10-17',
                  'Statement': [
                    {
                      'Action': [
                        'logs:CreateLogStream',
                        'logs:PutLogEvents'
                      ],
                      'Effect': 'Allow',
                      'Resource': '*'
                    }
                  ]
                }




import boto3

"""
Remediate Evident Signature:

AWS:IAM-016 Custom IAM Policy Grants Too Many Privileges

Description:

Best practice is to not create a custom IAM policy with administrator access. 
If administrator access grant is needed, use the default AdministratorAccess policy

Required permissions:

- iam:CreatePolicyVersion

Sample IAM Policy:

{
  "Version": "2012-10-17",
  "Statement": [
    {
      "Sid": "Stmt1507759700000",
      "Effect": "Allow",
      "Action": [
        "iam:CreatePolicyVersion"
      ],
      "Resource": [
        "*"
      ]
    }
  ]
}

Remediation result:
- Modify custom policy with a placeholder policy with minimum access
{
    "Version": "2012-10-17",
    "Statement": [
        {
            "Sid": "VisualEditor0",
            "Effect": "Allow",
            "Action": [
                "sts:getCallerIdentity"
            ],
            "Resource": "*"
        }
    ]
}



"""

def remediate(session, alert, lambda_context):
    print('Remediation for AWS:Iam-016 started.')

    new_policy = "{\n    \"Version\": \"2012-10-17\",\n    \"Statement\": [\n        {\n            \"Sid\": \"VisualEditor0\",\n            \"Effect\": \"Allow\",\n            \"Action\": [\n                \"sts:getCallerIdentity\"\n            ],\n            \"Resource\": \"*\"\n        }\n    ]\n}"
    

    client = session.client('iam')
    print('Modifying policy: {}'.format(alert['resource_id']))
    
    try:
        resp = client.create_policy_version(
            PolicyArn = alert['metadata']['arn'],
            PolicyDocument = new_policy,
            SetAsDefault = True
            )

    except Exception as e:
        raise e
    
    print('Policy Update complete')
    return 0



"""
Remediate Evident Signature:

AWS:RDS-010 RDS Multi-AZ Check

Description:

Amazon RDS Multi-AZ deployments provide enhanced availability and durability for Database (DB) Instances,
making them a natural fit for production database workloads. When you provision a Multi-AZ DB Instance,
Amazon RDS automatically creates a primary DB Instance and synchronously replicates the data to a standby
instance in a different Availability Zone (AZ). Each AZ runs on its own physically distinct, independent
infrastructure, and is engineered to be highly reliable. In case of an infrastructure failure, Amazon RDS
performs an automatic failover to the standby (or to a read replica in the case of Amazon Aurora), so that
you can resume database operations as soon as the failover is complete. Since the endpoint for your DB
instance remains the same after a failover, your application can resume database operation without the
need for manual administrative intervention.

Required Permissions:

- rds:DescribeDBInstances
- rds:ModifyDBInstance

Sample IAM Policy:

{
  "Version": "2012-10-17",
  "Statement": [
    {
      "Sid": "RDSPermissions",
      "Action": [
        "rds:DescribeDBInstances",
        "rds:ModifyDBInstance"
      ],
      "Effect": "Allow",
      "Resource": "*"
    }
  ]
}
"""

import boto3
from botocore.exceptions import ClientError


def remediate(session, alert, lambda_context):
  """
  Main Function invoked by index.py
  """

  instance_id = alert['resource_id']
  region = alert['region']

  rds = session.client('rds', region_name=region)

  try:
    db_instance = rds.describe_db_instances(DBInstanceIdentifier = instance_id)['DBInstances']
  except ClientError as e:
    print(e.response['Error']['Message'])
    return

  try:
    multi_az = db_instance[0]['MultiAZ']
  except (KeyError, IndexError):
    pass

  else:

    if multi_az != True:

      try:
        result = rds.modify_db_instance(
          DBInstanceIdentifier = instance_id,
          ApplyImmediately = True,
          MultiAZ = True
        )
      except ClientError as e:
        print(e.response['Error']['Message'])
      else:
        print('Enabled \'MultiAZ\' for RDS instance {}.'.format(instance_id))

  return




from setuptools import setup

with open("README.rst", "r") as fh:
    long_description = fh.read()

setup(
    name="remediate",
    version="0.0.16",
    packages=['remediate','remediate/runbook'],
    include_package_data=True,
    install_requires=['boto3'],
    python_requires='~=3.7',
    license='MIT',
    author='cloudcraeft',
    author_email='cloudcraeft@outloook.com',
    long_description=long_description,
    url='https://github.com/cloudcraeft/remediate',
    description='integrate previously written remediation code into Demisto',
)



"""
Remediate Evident Signature:

AWS:ELB-012 ELB (Classic) Cross-Zone Load Balancing

Description:

Cross-zone load balancing reduces the need to maintain equivalent numbers of instances in each enabled
Availability Zone, and improves your application's ability to handle the loss of one or more instances.
However, we still recommend that you maintain approximately equivalent numbers of instances in each
enabled Availability Zone for higher fault tolerance.

Required Permissions:

- elasticloadbalancing:DescribeLoadBalancerAttributes
- elasticloadbalancing:ModifyLoadBalancerAttributes

Sample IAM Policy:

{
  "Version": "2012-10-17",
  "Statement": [
    {
      "Sid": "ELBPermissions",
      "Action": [
        "elasticloadbalancing:DescribeLoadBalancerAttributes",
        "elasticloadbalancing:ModifyLoadBalancerAttributes"
      ],
      "Effect": "Allow",
      "Resource": "*"
    }
  ]
}
"""

import boto3
from botocore.exceptions import ClientError


def remediate(session, alert, lambda_context):
  """
  Main Function invoked by index.py
  """

  elb_name = alert['resource_id']
  region   = alert['region']

  elb = session.client('elb', region_name=region)

  try:
    attribs = elb.describe_load_balancer_attributes(LoadBalancerName=elb_name)['LoadBalancerAttributes']
  except ClientError as e:
    print(e.response['Error']['Message'])
    return

  cross_zone = attribs['CrossZoneLoadBalancing']

  if cross_zone['Enabled'] != True:
    result = enable_cross_zone(elb, elb_name)

  return


def enable_cross_zone(elb, elb_name):
  """
  Enable ELB (Classic) Cross-Zone Load Balancing
  """

  try:
    result = elb.modify_load_balancer_attributes(
      LoadBalancerName = elb_name,
      LoadBalancerAttributes = {
        'CrossZoneLoadBalancing': {
          'Enabled': True
        }
      }
    )
  except ClientError as e:
    print(e.response['Error']['Message'])
  else:
    print('Enabled Cross-Zone Load Balancing for ELB {}.'.format(elb_name))

  return




"""
Remediate Evident Signature:

AWS:EC2-036 Public AMI Detected

Description:

Publicly shared AMIs pose a risk in exposing sensitive data.

Required Permissions:

- ec2:DescribeImageAttribute
- ec2:ModifyImageAttribute

Sample IAM Policy:

{
    "Version": "2012-10-17",
    "Statement": [
        {
            "Sid": "EC2Permissions",
            "Effect": "Allow",
            "Action": [
                "ec2:DescribeImageAttribute",
                "ec2:ModifyImageAttribute"
            ],
            "Resource": "*"
        }
    ]
}
"""

import boto3
from botocore.exceptions import ClientError


def remediate(session, alert, lambda_context):
  """
  Main Function invoked by index.py
  """

  image_id = alert['resource_id']
  region   = alert['region']

  ec2  = session.client('ec2', region_name=region)

  try:
    attribute = ec2.describe_image_attribute(ImageId=image_id, Attribute='launchPermission')
    launch_permissions = attribute['LaunchPermissions']
  except ClientError as e:
    print(e.response['Error']['Message'])
    return

  # Check for 'Public' permissions
  if launch_permissions == [{'Group': 'all'}]:
    response = set_ami_to_private(ec2, image_id)

  return


def set_ami_to_private(ec2, image_id):
  """
  Toggle the AMI to private from public by removing 
  {'Group': 'all'} from AMI's LaunchPermissions
  """

  try:
    response = ec2.modify_image_attribute(
      ImageId = image_id,
      Attribute = 'launchPermission',
      LaunchPermission = {
        'Remove': [{'Group': 'all'}]
      }
    )

    print('Removed "Public" LaunchPermission from AMI {}.'.format(image_id))

  except ClientError as e:
    print(e.response['Error']['Message'])

  return




"""
Remediate Evident Signature:

AWS:VPC-020 VPC Flow Logs Not Enabled

Description:

VPC Flow Logs allows an organization to troubleshoot connectivity and security issues in a VPC.
Best security practice is to enable VPC Flow Logs on all VPCs.

Required Permissions:

- iam:PutRolePolicy
- iam:CreateRole
- iam:GetRole
- logs:CreateLogGroup
- logs:PutRetentionPolicy
- ec2:CreateFlowLogs

Sample IAM Policy:

{
  "Version": "2012-10-17",
  "Statement": [
    {
      "Sid": "IAMPermissions",
      "Action": [
        "iam:PutRolePolicy",
        "iam:CreateRole",
        "iam:GetRole"
      ],
      "Effect": "Allow",
      "Resource": "*"
    },
    {
      "Sid": "CloudWatchLogsPermissions",
      "Action": [
        "logs:CreateLogGroup",
        "logs:PutRetentionPolicy"
      ],
      "Effect": "Allow",
      "Resource": "*"
    },
    {
      "Sid": "EC2Permissions",
      "Action": [
        "ec2:CreateFlowLogs"
      ],
      "Effect": "Allow",
      "Resource": "*"
    }
  ]
}
"""

import json
import boto3
from botocore.exceptions import ClientError

# Options:
#
# AWS default IAM Role name for Flow Logs
#
default_role_name   = 'flowlogsRole'
default_policy_name = 'flowlogsPolicy'


def remediate(session, alert, lambda_context):
  """
  Main Function invoked by index.py
  """

  vpc_id = alert['resource_id']
  region = alert['region']

  iam  = session.client('iam', region_name=region)
  logs = session.client('logs', region_name=region)
  ec2  = session.client('ec2', region_name=region)

  # Create IAM Role and Policy
  role_arn = new_iam_role(iam)

  # Create CloudWatch Logs group
  log_group_name = new_log_group(logs, vpc_id) if (role_arn != 'fail') else 'fail'

  # Enable VPC Flow Logs
  flow_id = new_flow_logs(ec2, vpc_id, log_group_name, role_arn) if (log_group_name != 'fail') else 'fail'

  if flow_id != 'fail':
    print('VPC Flow Logs enabled for VPC: {}'.format(vpc_id))
  else:
    print('Failed to enable VPC Flow Logs for VPC: {}'.format(vpc_id))

  return


def new_iam_role(iam):
  """
  Create new IAM Role and Inline Policy
  """

  try:
    role = iam.create_role(
      Path = '/',
      RoleName = default_role_name,
      AssumeRolePolicyDocument = json.dumps(Templates.RolePolicy)
    )
    role_arn  = role['Role']['Arn']

    print('New IAM Role created: {}'.format(role_arn))

  except ClientError as e:
    if e.response['Error']['Code'] == 'EntityAlreadyExists':
      role = iam.get_role(
        RoleName = default_role_name
      )
      role_arn  = role['Role']['Arn']

      print('Using existing IAM Role: {}'.format(role_arn))

    else:
      print(e.response['Error']['Message'])
      return 'fail'

  try:
    policy = iam.put_role_policy(
      RoleName = default_role_name,
      PolicyName = default_policy_name,
      PolicyDocument = json.dumps(Templates.LogsPolicy)
    )
  except ClientError as e:
    print(e.response['Error']['Message'])
    return 'fail'

  return role_arn


def new_log_group(logs, vpc_id):
  """
  Create CloudWatch Logs group
  """

  log_group_name = 'flowlogsGroup' + '-' + vpc_id

  try:
    group = logs.create_log_group(
      logGroupName = log_group_name
    )

    print('New CloudWatch Logs group created: {}'.format(log_group_name))

  except ClientError as e:
    if e.response['Error']['Code'] == 'ResourceAlreadyExistsException':
      print('Using existing CloudWatch Log group: {}'.format(log_group_name))
    else:
      print(e.response['Error']['Message'])
      return 'fail'

  try:
    retention = logs.put_retention_policy(
      logGroupName = log_group_name,
      retentionInDays = 30
    )
  except ClientError as e:
    print(e.response['Error']['Message'])

  return log_group_name
  

def new_flow_logs(ec2, vpc_id, log_group_name, role_arn):
  """
  Enable VPC Flow Logs
  """

  try:
    flow_logs = ec2.create_flow_logs(
      ResourceIds = [vpc_id],
      ResourceType = 'VPC',
      TrafficType = 'ALL',
      LogGroupName = log_group_name,
      DeliverLogsPermissionArn = role_arn
    )
  except ClientError as e:
    print(e.response['Error']['Message'])
    return 'fail'

  flow_id = flow_logs['FlowLogIds'][0]

  print('New Flow Logs created: {}'.format(flow_id))

  return flow_id


class Templates:

  RolePolicy =  {
                  'Version': '2012-10-17',
                  'Statement': [
                    {
                      'Sid': '',
                      'Effect': 'Allow',
                      'Principal': {
                        'Service': 'vpc-flow-logs.amazonaws.com'
                      },
                      'Action': 'sts:AssumeRole'
                    }
                  ]
                }

  LogsPolicy =  {
                  'Version': '2012-10-17',
                  'Statement': [
                    {
                      'Action': [
                        'logs:CreateLogGroup',
                        'logs:CreateLogStream',
                        'logs:DescribeLogGroups',
                        'logs:DescribeLogStreams',
                        'logs:PutLogEvents'
                      ],
                      'Effect': 'Allow',
                      'Resource': '*'
                    }
                  ]
                }




"""
Remediate Evident Signature:

AWS:RDS-007 RDS Snapshot with Public Permissions

Description:

If you set DB Snapshot Visibility to Public, all AWS accounts can restore a DB instance from your
DB snapshot and have access to your data. Best practice is to not share any DB snapshots that contain
private information as Public.

Required Permissions:

- rds:DescribeDBSnapshotAttributes
- rds:ModifyDBSnapshotAttribute

Sample IAM Policy:

{
    "Version": "2012-10-17",
    "Statement": [
        {
            "Sid": "RDSPermissions",
            "Effect": "Allow",
            "Action": [
                "rds:DescribeDBSnapshotAttributes",
                "rds:ModifyDBSnapshotAttribute"
            ],
            "Resource": [
                "*"
            ]
        }
    ]
}
"""

import boto3
from botocore.exceptions import ClientError


def remediate(session, alert, lambda_context):
  """
  Main Function invoked by index.py
  """

  snapshot_id = alert['resource_id']
  region = alert['region']

  rds = session.client('rds', region_name=region)

  try:
    attributes = rds.describe_db_snapshot_attributes(DBSnapshotIdentifier=snapshot_id)['DBSnapshotAttributesResult']['DBSnapshotAttributes']
  except ClientError as e:
    print(e.response['Error']['Message'])
    return
    
  public = False

  for attrib in attributes:
    if attrib['AttributeName'] == 'restore':
      try:
        if (attrib['AttributeValues'][0] == 'all'):
          public = True
      except IndexError:
          continue
    else:
      print('Unable to find public attribute for RDS snapshot {}.'.format(snapshot_id))

  if public == True:

    snap_args = {
      'DBSnapshotIdentifier' : snapshot_id,
      'AttributeName' : 'restore',
      'ValuesToRemove' : [ 'all' ]
    }

    try:
      results = rds.modify_db_snapshot_attribute(**snap_args)

      print('Removed public attribute from RDS snapshot {}.'.format(snapshot_id))

    except ClientError as e:
      print(e.response['Error']['Message'])

  return



"""
Remediate Evident Signature:

AWS:EC2-031 Unused Security Groups

Description:

Unused security groups pose a risk against your AWS Infrastructure, as they are often unmaintained,
legacy, or improperly created. Failing to remove these unused security groups could lead to inadvertent
use of them, creating open attack surfaces in your infrastructure.

Required Permissions:

- ec2:DeleteSecurityGroup
- ec2:DescribeSecurityGroups
- lambda:ListFunctions

Sample IAM Policy:

{
    "Version": "2012-10-17",
    "Statement": [
        {
            "Sid": "EC2LambdaPermissions",
            "Effect": "Allow",
            "Action": [
                "ec2:DeleteSecurityGroup",
                "ec2:DescribeSecurityGroups",
                "lambda:ListFunctions"
            ],
            "Resource": [
                "*"
            ]
        }
    ]
}
"""

import boto3
from botocore.exceptions import ClientError


def remediate(session, alert, lambda_context):
  """
  Main Function invoked by index.py
  """

  sg_id  = alert['resource_id']
  region = alert['region']

  ec2 = session.client('ec2', region_name=region)
  lam = session.client('lambda', region_name=region)

  # Check to see if the security group is tied to a Lambda function
  try:
    functions = lam.list_functions()['Functions']
  except ClientError as e:
    print(e.response['Error']['Message'])
    return

  check = False

  for function in functions:
    if check == True:
      print('Security group {} is tied to a Lambda function. No remediation performed.'.format(sg_id))
      return

    try:
      vpc_config = function['VpcConfig']
    except KeyError:
      continue
    else:
      security_groups = vpc_config['SecurityGroupIds']

      for group in security_groups:
        if group == sg_id:
          check = True

  # Remediate (security group is not tied to a Lambda function)
  result = delete_unused_sg(ec2, sg_id)

  return


def delete_unused_sg(ec2, sg_id):
  """
  Delete Unused Security Group
  """

  try:
    result = ec2.delete_security_group(GroupId=sg_id)
  except ClientError as e:
    print(e.response['Error']['Message'])
  else:
    print('Removed unused security group {}.'.format(sg_id))

  return




"""
Remediate Evident Signature:

AWS:CLT-005 Log file validation not enabled for CloudTrail Log File

Description:

The CloudTrail log file validation creates a digest for each log file written to S3. These digests
can be used to determine whether a log file was changed, deleted, or unchanged after CloudTrail delivered
the file.

Required Permissions:

- cloudtrail:DescribeTrails
- cloudtrail:UpdateTrail

Sample IAM Policy:

{
    "Version": "2012-10-17",
    "Statement": [
        {
            "Sid": "CloudTrailPermissions",
            "Effect": "Allow",
            "Action": [
                "cloudtrail:DescribeTrails",
                "cloudtrail:UpdateTrail"
            ],
            "Resource": [
                "*"
            ]
        }
    ]
}
"""

import boto3
from botocore.exceptions import ClientError


def remediate(session, alert, lambda_context):
  """
  Main Function invoked by index.py
  """

  trail_name  = alert['resource_id']
  region      = alert['region']

  clt = session.client('cloudtrail', region_name=region)

  try:
    trail = clt.describe_trails(trailNameList=[ trail_name ], includeShadowTrails=False)['trailList']
  except ClientError as e:
    print(e.response['Error']['Message'])
    return

  if trail[0]['LogFileValidationEnabled'] != True:
    result = enable_validation(clt, trail_name)

  return


def enable_validation(clt, trail_name):
  """
  Enable CloudTrail Log File Validation
  """

  try:
    result = clt.update_trail(Name=trail_name, EnableLogFileValidation=True)
  except ClientError as e:
    print(e.response['Error']['Message'])
  else:
    print('Enabled log file validation for Trail {}.'.format(trail_name))

  return

